@@ -4,13 +4,19 @@ namespace Illuminate\Cache;
 
 use Illuminate\Contracts\Cache\LockProvider;
 use Illuminate\Contracts\Redis\Factory as Redis;
+use Illuminate\Redis\Connections\PhpRedisClusterConnection;
 use Illuminate\Redis\Connections\PhpRedisConnection;
+use Illuminate\Redis\Connections\PredisClusterConnection;
 use Illuminate\Redis\Connections\PredisConnection;
 use Illuminate\Support\LazyCollection;
 use Illuminate\Support\Str;
 
 class RedisStore extends TaggableStore implements LockProvider
 {
+    use RetrievesMultipleKeys {
+        putMany as private putManyAlias;
+    }
+
     /**
      * The Redis factory implementation.
      *
@@ -57,14 +63,16 @@ class RedisStore extends TaggableStore implements LockProvider
     /**
      * Retrieve an item from the cache by key.
      *
-     * @param  string|array  $key
+     * @param  string  $key
      * @return mixed
      */
     public function get($key)
     {
-        $value = $this->connection()->get($this->prefix.$key);
+        $connection = $this->connection();
 
-        return ! is_null($value) ? $this->unserialize($value) : null;
+        $value = $connection->get($this->prefix.$key);
+
+        return ! is_null($value) ? $this->connectionAwareUnserialize($value, $connection) : null;
     }
 
     /**
@@ -83,12 +91,14 @@ class RedisStore extends TaggableStore implements LockProvider
 
         $results = [];
 
-        $values = $this->connection()->mget(array_map(function ($key) {
+        $connection = $this->connection();
+
+        $values = $connection->mget(array_map(function ($key) {
             return $this->prefix.$key;
         }, $keys));
 
         foreach ($values as $index => $value) {
-            $results[$keys[$index]] = ! is_null($value) ? $this->unserialize($value) : null;
+            $results[$keys[$index]] = ! is_null($value) ? $this->connectionAwareUnserialize($value, $connection) : null;
         }
 
         return $results;
@@ -104,8 +114,10 @@ class RedisStore extends TaggableStore implements LockProvider
      */
     public function put($key, $value, $seconds)
     {
-        return (bool) $this->connection()->setex(
-            $this->prefix.$key, (int) max(1, $seconds), $this->serialize($value)
+        $connection = $this->connection();
+
+        return (bool) $connection->setex(
+            $this->prefix.$key, (int) max(1, $seconds), $this->connectionAwareSerialize($value, $connection)
         );
     }
 
@@ -118,25 +130,33 @@ class RedisStore extends TaggableStore implements LockProvider
      */
     public function putMany(array $values, $seconds)
     {
+        $connection = $this->connection();
+
+        // Cluster connections do not support writing multiple values if the keys hash differently...
+        if ($connection instanceof PhpRedisClusterConnection ||
+            $connection instanceof PredisClusterConnection) {
+            return $this->putManyAlias($values, $seconds);
+        }
+
         $serializedValues = [];
 
         foreach ($values as $key => $value) {
             $serializedValues[$this->prefix.$key] = $this->serialize($value);
         }
 
-        $this->connection()->multi();
+        $connection->multi();
 
         $manyResult = null;
 
         foreach ($serializedValues as $key => $value) {
-            $result = (bool) $this->connection()->setex(
+            $result = (bool) $connection->setex(
                 $key, (int) max(1, $seconds), $value
             );
 
             $manyResult = is_null($manyResult) ? $result : $result && $manyResult;
         }
 
-        $this->connection()->exec();
+        $connection->exec();
 
         return $manyResult ?: false;
     }
@@ -151,10 +171,10 @@ class RedisStore extends TaggableStore implements LockProvider
      */
     public function add($key, $value, $seconds)
     {
-        $lua = "return redis.call('exists',KEYS[1])<1 and redis.call('setex',KEYS[1],ARGV[2],ARGV[1])";
+        $connection = $this->connection();
 
-        return (bool) $this->connection()->eval(
-            $lua, 1, $this->prefix.$key, $this->serialize($value), (int) max(1, $seconds)
+        return (bool) $connection->eval(
+            LuaScripts::add(), 1, $this->prefix.$key, $this->pack($value, $connection), (int) max(1, $seconds)
         );
     }
 
@@ -191,7 +211,9 @@ class RedisStore extends TaggableStore implements LockProvider
      */
     public function forever($key, $value)
     {
-        return (bool) $this->connection()->set($this->prefix.$key, $this->serialize($value));
+        $connection = $this->connection();
+
+        return (bool) $connection->set($this->prefix.$key, $this->connectionAwareSerialize($value, $connection));
     }
 
     /**
@@ -292,10 +314,15 @@ class RedisStore extends TaggableStore implements LockProvider
             default => '',
         };
 
+        $defaultCursorValue = match (true) {
+            $connection instanceof PhpRedisConnection && version_compare(phpversion('redis'), '6.1.0', '>=') => null,
+            default => '0',
+        };
+
         $prefix = $connectionPrefix.$this->getPrefix();
 
-        return LazyCollection::make(function () use ($connection, $chunkSize, $prefix) {
-            $cursor = $defaultCursorValue = '0';
+        return (new LazyCollection(function () use ($connection, $chunkSize, $prefix, $defaultCursorValue) {
+            $cursor = $defaultCursorValue;
 
             do {
                 [$cursor, $tagsChunk] = $connection->scan(
@@ -317,7 +344,7 @@ class RedisStore extends TaggableStore implements LockProvider
                     yield $tag;
                 }
             } while (((string) $cursor) !== $defaultCursorValue);
-        })->map(fn (string $tagKey) => Str::match('/^'.preg_quote($prefix, '/').'tag:(.*):entries$/', $tagKey));
+        }))->map(fn (string $tagKey) => Str::match('/^'.preg_quote($prefix, '/').'tag:(.*):entries$/', $tagKey));
     }
 
     /**
@@ -392,7 +419,29 @@ class RedisStore extends TaggableStore implements LockProvider
      */
     public function setPrefix($prefix)
     {
-        $this->prefix = ! empty($prefix) ? $prefix.':' : '';
+        $this->prefix = $prefix;
+    }
+
+    /**
+     * Prepare a value to be used with the Redis cache store when used by eval scripts.
+     *
+     * @param  mixed  $value
+     * @param  \Illuminate\Redis\Connections\Connection  $connection
+     * @return mixed
+     */
+    protected function pack($value, $connection)
+    {
+        if ($connection instanceof PhpRedisConnection) {
+            if ($connection->serialized()) {
+                return $connection->pack([$value])[0];
+            }
+
+            if ($connection->compressed()) {
+                return $connection->pack([$this->serialize($value)])[0];
+            }
+        }
+
+        return $this->serialize($value);
     }
 
     /**
@@ -403,7 +452,18 @@ class RedisStore extends TaggableStore implements LockProvider
      */
     protected function serialize($value)
     {
-        return is_numeric($value) && ! in_array($value, [INF, -INF]) && ! is_nan($value) ? $value : serialize($value);
+        return $this->shouldBeStoredWithoutSerialization($value) ? $value : serialize($value);
+    }
+
+    /**
+     * Determine if the given value should be stored as plain value.
+     *
+     * @param  mixed  $value
+     * @return bool
+     */
+    protected function shouldBeStoredWithoutSerialization($value): bool
+    {
+        return is_numeric($value) && ! in_array($value, [INF, -INF]) && ! is_nan($value);
     }
 
     /**
@@ -416,4 +476,36 @@ class RedisStore extends TaggableStore implements LockProvider
     {
         return is_numeric($value) ? $value : unserialize($value);
     }
+
+    /**
+     * Handle connection specific considerations when a value needs to be serialized.
+     *
+     * @param  mixed  $value
+     * @param  \Illuminate\Redis\Connections\Connection  $connection
+     * @return mixed
+     */
+    protected function connectionAwareSerialize($value, $connection)
+    {
+        if ($connection instanceof PhpRedisConnection && $connection->serialized()) {
+            return $value;
+        }
+
+        return $this->serialize($value);
+    }
+
+    /**
+     * Handle connection specific considerations when a value needs to be unserialized.
+     *
+     * @param  mixed  $value
+     * @param  \Illuminate\Redis\Connections\Connection  $connection
+     * @return mixed
+     */
+    protected function connectionAwareUnserialize($value, $connection)
+    {
+        if ($connection instanceof PhpRedisConnection && $connection->serialized()) {
+            return $value;
+        }
+
+        return $this->unserialize($value);
+    }
 }
