@@ -2,7 +2,11 @@
 
 namespace Illuminate\Database\Migrations;
 
-use Illuminate\Console\OutputStyle;
+use Closure;
+use Illuminate\Console\View\Components\BulletList;
+use Illuminate\Console\View\Components\Info;
+use Illuminate\Console\View\Components\Task;
+use Illuminate\Console\View\Components\TwoColumnDetail;
 use Illuminate\Contracts\Events\Dispatcher;
 use Illuminate\Database\ConnectionResolverInterface as Resolver;
 use Illuminate\Database\Events\MigrationEnded;
@@ -14,6 +18,8 @@ use Illuminate\Filesystem\Filesystem;
 use Illuminate\Support\Arr;
 use Illuminate\Support\Collection;
 use Illuminate\Support\Str;
+use ReflectionClass;
+use Symfony\Component\Console\Output\OutputInterface;
 
 class Migrator
 {
@@ -45,6 +51,13 @@ class Migrator
      */
     protected $resolver;
 
+    /**
+     * The custom connection resolver callback.
+     *
+     * @var \Closure|null
+     */
+    protected static $connectionResolverCallback;
+
     /**
      * The name of the default connection.
      *
@@ -55,17 +68,31 @@ class Migrator
     /**
      * The paths to all of the migration files.
      *
-     * @var array
+     * @var string[]
      */
     protected $paths = [];
 
+    /**
+     * The paths that have already been required.
+     *
+     * @var array<string, \Illuminate\Database\Migrations\Migration|null>
+     */
+    protected static $requiredPathCache = [];
+
     /**
      * The output interface implementation.
      *
-     * @var \Illuminate\Console\OutputStyle
+     * @var \Symfony\Component\Console\Output\OutputInterface
      */
     protected $output;
 
+    /**
+     * The pending migrations to skip.
+     *
+     * @var list<string>
+     */
+    protected static $withoutMigrations = [];
+
     /**
      * Create a new migrator instance.
      *
@@ -75,11 +102,12 @@ class Migrator
      * @param  \Illuminate\Contracts\Events\Dispatcher|null  $dispatcher
      * @return void
      */
-    public function __construct(MigrationRepositoryInterface $repository,
-                                Resolver $resolver,
-                                Filesystem $files,
-                                Dispatcher $dispatcher = null)
-    {
+    public function __construct(
+        MigrationRepositoryInterface $repository,
+        Resolver $resolver,
+        Filesystem $files,
+        ?Dispatcher $dispatcher = null,
+    ) {
         $this->files = $files;
         $this->events = $dispatcher;
         $this->resolver = $resolver;
@@ -89,9 +117,9 @@ class Migrator
     /**
      * Run the pending migrations at a given path.
      *
-     * @param  array|string  $paths
-     * @param  array  $options
-     * @return array
+     * @param  string[]|string  $paths
+     * @param  array<string, mixed>  $options
+     * @return string[]
      */
     public function run($paths = [], array $options = [])
     {
@@ -115,23 +143,39 @@ class Migrator
     /**
      * Get the migration files that have not yet run.
      *
-     * @param  array  $files
-     * @param  array  $ran
-     * @return array
+     * @param  string[]  $files
+     * @param  string[]  $ran
+     * @return string[]
      */
     protected function pendingMigrations($files, $ran)
     {
-        return Collection::make($files)
-                ->reject(function ($file) use ($ran) {
-                    return in_array($this->getMigrationName($file), $ran);
-                })->values()->all();
+        $migrationsToSkip = $this->migrationsToSkip();
+
+        return (new Collection($files))
+            ->reject(fn ($file) => in_array($migrationName = $this->getMigrationName($file), $ran) ||
+                in_array($migrationName, $migrationsToSkip)
+            )
+            ->values()
+            ->all();
+    }
+
+    /**
+     * Get list of pending migrations to skip.
+     *
+     * @return list<string>
+     */
+    protected function migrationsToSkip()
+    {
+        return (new Collection(self::$withoutMigrations))
+            ->map($this->getMigrationName(...))
+            ->all();
     }
 
     /**
      * Run an array of migrations.
      *
-     * @param  array  $migrations
-     * @param  array  $options
+     * @param  string[]  $migrations
+     * @param  array<string, mixed>  $options
      * @return void
      */
     public function runPending(array $migrations, array $options = [])
@@ -142,7 +186,7 @@ class Migrator
         if (count($migrations) === 0) {
             $this->fireMigrationEvent(new NoPendingMigrations('up'));
 
-            $this->note('<info>Nothing to migrate.</info>');
+            $this->write(Info::class, 'Nothing to migrate');
 
             return;
         }
@@ -156,7 +200,9 @@ class Migrator
 
         $step = $options['step'] ?? false;
 
-        $this->fireMigrationEvent(new MigrationsStarted);
+        $this->fireMigrationEvent(new MigrationsStarted('up', $options));
+
+        $this->write(Info::class, 'Running migrations.');
 
         // Once we have the array of migrations, we will spin through them and run the
         // migrations "up" so the changes are made to the databases. We'll then log
@@ -169,7 +215,9 @@ class Migrator
             }
         }
 
-        $this->fireMigrationEvent(new MigrationsEnded);
+        $this->fireMigrationEvent(new MigrationsEnded('up', $options));
+
+        $this->output?->writeln('');
     }
 
     /**
@@ -185,36 +233,28 @@ class Migrator
         // First we will resolve a "real" instance of the migration class from this
         // migration file name. Once we have the instances we can run the actual
         // command such as "up" or "down", or we can just simulate the action.
-        $migration = $this->resolve(
-            $name = $this->getMigrationName($file)
-        );
+        $migration = $this->resolvePath($file);
+
+        $name = $this->getMigrationName($file);
 
         if ($pretend) {
             return $this->pretendToRun($migration, 'up');
         }
 
-        $this->note("<comment>Migrating:</comment> {$name}");
-
-        $startTime = microtime(true);
-
-        $this->runMigration($migration, 'up');
-
-        $runTime = round(microtime(true) - $startTime, 2);
+        $this->write(Task::class, $name, fn () => $this->runMigration($migration, 'up'));
 
         // Once we have run a migrations class, we will log that it was run in this
         // repository so that we don't try to run it next time we do a migration
         // in the application. A migration repository keeps the migrate order.
         $this->repository->log($name, $batch);
-
-        $this->note("<info>Migrated:</info>  {$name} ({$runTime} seconds)");
     }
 
     /**
      * Rollback the last migration operation.
      *
-     * @param  array|string  $paths
-     * @param  array  $options
-     * @return array
+     * @param  string[]|string  $paths
+     * @param  array<string, mixed>  $options
+     * @return string[]
      */
     public function rollback($paths = [], array $options = [])
     {
@@ -226,18 +266,20 @@ class Migrator
         if (count($migrations) === 0) {
             $this->fireMigrationEvent(new NoPendingMigrations('down'));
 
-            $this->note('<info>Nothing to rollback.</info>');
+            $this->write(Info::class, 'Nothing to rollback.');
 
             return [];
         }
 
-        return $this->rollbackMigrations($migrations, $paths, $options);
+        return tap($this->rollbackMigrations($migrations, $paths, $options), function () {
+            $this->output?->writeln('');
+        });
     }
 
     /**
      * Get the migrations for a rollback operation.
      *
-     * @param  array  $options
+     * @param  array<string, mixed>  $options
      * @return array
      */
     protected function getMigrationsForRollback(array $options)
@@ -246,6 +288,10 @@ class Migrator
             return $this->repository->getMigrations($steps);
         }
 
+        if (($batch = $options['batch'] ?? 0) > 0) {
+            return $this->repository->getMigrationsByBatch($batch);
+        }
+
         return $this->repository->getLast();
     }
 
@@ -253,9 +299,9 @@ class Migrator
      * Rollback the given migrations.
      *
      * @param  array  $migrations
-     * @param  array|string  $paths
-     * @param  array  $options
-     * @return array
+     * @param  string[]|string  $paths
+     * @param  array<string, mixed>  $options
+     * @return string[]
      */
     protected function rollbackMigrations(array $migrations, $paths, array $options)
     {
@@ -263,7 +309,9 @@ class Migrator
 
         $this->requireFiles($files = $this->getMigrationFiles($paths));
 
-        $this->fireMigrationEvent(new MigrationsStarted);
+        $this->fireMigrationEvent(new MigrationsStarted('down', $options));
+
+        $this->write(Info::class, 'Rolling back migrations.');
 
         // Next we will run through all of the migrations and call the "down" method
         // which will reverse each migration in order. This getLast method on the
@@ -272,7 +320,7 @@ class Migrator
             $migration = (object) $migration;
 
             if (! $file = Arr::get($files, $migration->migration)) {
-                $this->note("<fg=red>Migration not found:</> {$migration->migration}");
+                $this->write(TwoColumnDetail::class, $migration->migration, '<fg=yellow;options=bold>Migration not found</>');
 
                 continue;
             }
@@ -285,7 +333,7 @@ class Migrator
             );
         }
 
-        $this->fireMigrationEvent(new MigrationsEnded);
+        $this->fireMigrationEvent(new MigrationsEnded('down', $options));
 
         return $rolledBack;
     }
@@ -293,7 +341,7 @@ class Migrator
     /**
      * Rolls all of the currently applied migrations back.
      *
-     * @param  array|string  $paths
+     * @param  string[]|string  $paths
      * @param  bool  $pretend
      * @return array
      */
@@ -305,19 +353,21 @@ class Migrator
         $migrations = array_reverse($this->repository->getRan());
 
         if (count($migrations) === 0) {
-            $this->note('<info>Nothing to rollback.</info>');
+            $this->write(Info::class, 'Nothing to rollback.');
 
             return [];
         }
 
-        return $this->resetMigrations($migrations, $paths, $pretend);
+        return tap($this->resetMigrations($migrations, Arr::wrap($paths), $pretend), function () {
+            $this->output?->writeln('');
+        });
     }
 
     /**
      * Reset the given migrations.
      *
-     * @param  array  $migrations
-     * @param  array  $paths
+     * @param  string[]  $migrations
+     * @param  string[]  $paths
      * @param  bool  $pretend
      * @return array
      */
@@ -326,9 +376,7 @@ class Migrator
         // Since the getRan method that retrieves the migration name just gives us the
         // migration name, we will format the names into objects with the name as a
         // property on the objects so that we can pass it to the rollback method.
-        $migrations = collect($migrations)->map(function ($m) {
-            return (object) ['migration' => $m];
-        })->all();
+        $migrations = (new Collection($migrations))->map(fn ($m) => (object) ['migration' => $m])->all();
 
         return $this->rollbackMigrations(
             $migrations, $paths, compact('pretend')
@@ -348,28 +396,20 @@ class Migrator
         // First we will get the file name of the migration so we can resolve out an
         // instance of the migration. Once we get an instance we can either run a
         // pretend execution of the migration or we can run the real migration.
-        $instance = $this->resolve(
-            $name = $this->getMigrationName($file)
-        );
+        $instance = $this->resolvePath($file);
 
-        $this->note("<comment>Rolling back:</comment> {$name}");
+        $name = $this->getMigrationName($file);
 
         if ($pretend) {
             return $this->pretendToRun($instance, 'down');
         }
 
-        $startTime = microtime(true);
-
-        $this->runMigration($instance, 'down');
-
-        $runTime = round(microtime(true) - $startTime, 2);
+        $this->write(Task::class, $name, fn () => $this->runMigration($instance, 'down'));
 
         // Once we have successfully run the migration "down" we will remove it from
         // the migration repository so it will be considered to have not been run
         // by the application then will be able to fire by any later operation.
         $this->repository->delete($migration);
-
-        $this->note("<info>Rolled back:</info>  {$name} ({$runTime} seconds)");
     }
 
     /**
@@ -385,11 +425,11 @@ class Migrator
             $migration->getConnection()
         );
 
-        $callback = function () use ($migration, $method) {
+        $callback = function () use ($connection, $migration, $method) {
             if (method_exists($migration, $method)) {
                 $this->fireMigrationEvent(new MigrationStarted($migration, $method));
 
-                $migration->{$method}();
+                $this->runMethod($connection, $migration, $method);
 
                 $this->fireMigrationEvent(new MigrationEnded($migration, $method));
             }
@@ -410,11 +450,20 @@ class Migrator
      */
     protected function pretendToRun($migration, $method)
     {
-        foreach ($this->getQueries($migration, $method) as $query) {
-            $name = get_class($migration);
+        $name = get_class($migration);
 
-            $this->note("<info>{$name}:</info> {$query['query']}");
+        $reflectionClass = new ReflectionClass($migration);
+
+        if ($reflectionClass->isAnonymous()) {
+            $name = $this->getMigrationName($reflectionClass->getFileName());
         }
+
+        $this->write(TwoColumnDetail::class, $name);
+
+        $this->write(
+            BulletList::class,
+            (new Collection($this->getQueries($migration, $method)))->map(fn ($query) => $query['query'])
+        );
     }
 
     /**
@@ -433,13 +482,34 @@ class Migrator
             $migration->getConnection()
         );
 
-        return $db->pretend(function () use ($migration, $method) {
+        return $db->pretend(function () use ($db, $migration, $method) {
             if (method_exists($migration, $method)) {
-                $migration->{$method}();
+                $this->runMethod($db, $migration, $method);
             }
         });
     }
 
+    /**
+     * Run a migration method on the given connection.
+     *
+     * @param  \Illuminate\Database\Connection  $connection
+     * @param  object  $migration
+     * @param  string  $method
+     * @return void
+     */
+    protected function runMethod($connection, $migration, $method)
+    {
+        $previousConnection = $this->resolver->getDefaultConnection();
+
+        try {
+            $this->resolver->setDefaultConnection($connection->getName());
+
+            $migration->{$method}();
+        } finally {
+            $this->resolver->setDefaultConnection($previousConnection);
+        }
+    }
+
     /**
      * Resolve a migration instance from a file.
      *
@@ -448,32 +518,68 @@ class Migrator
      */
     public function resolve($file)
     {
-        $class = Str::studly(implode('_', array_slice(explode('_', $file), 4)));
+        $class = $this->getMigrationClass($file);
 
         return new $class;
     }
 
+    /**
+     * Resolve a migration instance from a migration path.
+     *
+     * @param  string  $path
+     * @return object
+     */
+    protected function resolvePath(string $path)
+    {
+        $class = $this->getMigrationClass($this->getMigrationName($path));
+
+        if (class_exists($class) && realpath($path) == (new ReflectionClass($class))->getFileName()) {
+            return new $class;
+        }
+
+        $migration = static::$requiredPathCache[$path] ??= $this->files->getRequire($path);
+
+        if (is_object($migration)) {
+            return method_exists($migration, '__construct')
+                    ? $this->files->getRequire($path)
+                    : clone $migration;
+        }
+
+        return new $class;
+    }
+
+    /**
+     * Generate a migration class name based on the migration file name.
+     *
+     * @param  string  $migrationName
+     * @return string
+     */
+    protected function getMigrationClass(string $migrationName): string
+    {
+        return Str::studly(implode('_', array_slice(explode('_', $migrationName), 4)));
+    }
+
     /**
      * Get all of the migration files in a given path.
      *
      * @param  string|array  $paths
-     * @return array
+     * @return array<string, string>
      */
     public function getMigrationFiles($paths)
     {
-        return Collection::make($paths)->flatMap(function ($path) {
-            return Str::endsWith($path, '.php') ? [$path] : $this->files->glob($path.'/*_*.php');
-        })->filter()->values()->keyBy(function ($file) {
-            return $this->getMigrationName($file);
-        })->sortBy(function ($file, $key) {
-            return $key;
-        })->all();
+        return (new Collection($paths))
+            ->flatMap(fn ($path) => str_ends_with($path, '.php') ? [$path] : $this->files->glob($path.'/*_*.php'))
+            ->filter()
+            ->values()
+            ->keyBy(fn ($file) => $this->getMigrationName($file))
+            ->sortBy(fn ($file, $key) => $key)
+            ->all();
     }
 
     /**
      * Require in all the migration files in a given path.
      *
-     * @param  array  $files
+     * @param  string[]  $files
      * @return void
      */
     public function requireFiles(array $files)
@@ -508,13 +614,24 @@ class Migrator
     /**
      * Get all of the custom migration paths.
      *
-     * @return array
+     * @return string[]
      */
     public function paths()
     {
         return $this->paths;
     }
 
+    /**
+     * Set the pending migrations to skip.
+     *
+     * @param  list<string>  $migrations
+     * @return void
+     */
+    public static function withoutMigrations(array $migrations)
+    {
+        static::$withoutMigrations = $migrations;
+    }
+
     /**
      * Get the default connection name.
      *
@@ -525,6 +642,22 @@ class Migrator
         return $this->connection;
     }
 
+    /**
+     * Execute the given callback using the given connection as the default connection.
+     *
+     * @param  string  $name
+     * @param  callable  $callback
+     * @return mixed
+     */
+    public function usingConnection($name, callable $callback)
+    {
+        $previousConnection = $this->resolver->getDefaultConnection();
+
+        $this->setConnection($name);
+
+        return tap($callback(), fn () => $this->setConnection($previousConnection));
+    }
+
     /**
      * Set the default connection name.
      *
@@ -550,7 +683,26 @@ class Migrator
      */
     public function resolveConnection($connection)
     {
-        return $this->resolver->connection($connection ?: $this->connection);
+        if (static::$connectionResolverCallback) {
+            return call_user_func(
+                static::$connectionResolverCallback,
+                $this->resolver,
+                $connection ?: $this->connection
+            );
+        } else {
+            return $this->resolver->connection($connection ?: $this->connection);
+        }
+    }
+
+    /**
+     * Set a connection resolver callback.
+     *
+     * @param  \Closure  $callback
+     * @return void
+     */
+    public static function resolveConnectionsUsing(Closure $callback)
+    {
+        static::$connectionResolverCallback = $callback;
     }
 
     /**
@@ -590,6 +742,26 @@ class Migrator
         return $this->repository->repositoryExists();
     }
 
+    /**
+     * Determine if any migrations have been run.
+     *
+     * @return bool
+     */
+    public function hasRunAnyMigrations()
+    {
+        return $this->repositoryExists() && count($this->repository->getRan()) > 0;
+    }
+
+    /**
+     * Delete the migration repository data store.
+     *
+     * @return void
+     */
+    public function deleteRepository()
+    {
+        $this->repository->deleteRepository();
+    }
+
     /**
      * Get the file system instance.
      *
@@ -603,10 +775,10 @@ class Migrator
     /**
      * Set the output implementation that should be used by the console.
      *
-     * @param  \Illuminate\Console\OutputStyle  $output
+     * @param  \Symfony\Component\Console\Output\OutputInterface  $output
      * @return $this
      */
-    public function setOutput(OutputStyle $output)
+    public function setOutput(OutputInterface $output)
     {
         $this->output = $output;
 
@@ -614,15 +786,22 @@ class Migrator
     }
 
     /**
-     * Write a note to the console's output.
+     * Write to the console's output.
      *
-     * @param  string  $message
+     * @param  string  $component
+     * @param  array<int, string>|string  ...$arguments
      * @return void
      */
-    protected function note($message)
+    protected function write($component, ...$arguments)
     {
-        if ($this->output) {
-            $this->output->writeln($message);
+        if ($this->output && class_exists($component)) {
+            (new $component($this->output))->render(...$arguments);
+        } else {
+            foreach ($arguments as $argument) {
+                if (is_callable($argument)) {
+                    $argument();
+                }
+            }
         }
     }
 
@@ -634,8 +813,6 @@ class Migrator
      */
     public function fireMigrationEvent($event)
     {
-        if ($this->events) {
-            $this->events->dispatch($event);
-        }
+        $this->events?->dispatch($event);
     }
 }
