@@ -2,13 +2,18 @@
 
 namespace Illuminate\Notifications\Channels;
 
+use Illuminate\Config\Repository as ConfigRepository;
+use Illuminate\Container\Container;
 use Illuminate\Contracts\Mail\Factory as MailFactory;
 use Illuminate\Contracts\Mail\Mailable;
 use Illuminate\Contracts\Queue\ShouldQueue;
 use Illuminate\Mail\Markdown;
 use Illuminate\Notifications\Notification;
 use Illuminate\Support\Arr;
+use Illuminate\Support\Collection;
 use Illuminate\Support\Str;
+use Symfony\Component\Mailer\Header\MetadataHeader;
+use Symfony\Component\Mailer\Header\TagHeader;
 
 class MailChannel
 {
@@ -44,7 +49,7 @@ class MailChannel
      *
      * @param  mixed  $notifiable
      * @param  \Illuminate\Notifications\Notification  $notification
-     * @return void
+     * @return \Illuminate\Mail\SentMessage|null
      */
     public function send($notifiable, Notification $notification)
     {
@@ -59,7 +64,7 @@ class MailChannel
             return $message->send($this->mailer);
         }
 
-        $this->mailer->mailer($message->mailer ?? null)->send(
+        return $this->mailer->mailer($message->mailer ?? null)->send(
             $this->buildView($message),
             array_merge($message->data(), $this->additionalMessageData($notification)),
             $this->messageBuilder($notifiable, $notification, $message)
@@ -93,16 +98,53 @@ class MailChannel
             return $message->view;
         }
 
-        if (property_exists($message, 'theme') && ! is_null($message->theme)) {
-            $this->markdown->theme($message->theme);
-        }
-
         return [
-            'html' => $this->markdown->render($message->markdown, $message->data()),
-            'text' => $this->markdown->renderText($message->markdown, $message->data()),
+            'html' => $this->buildMarkdownHtml($message),
+            'text' => $this->buildMarkdownText($message),
         ];
     }
 
+    /**
+     * Build the HTML view for a Markdown message.
+     *
+     * @param  \Illuminate\Notifications\Messages\MailMessage  $message
+     * @return \Closure
+     */
+    protected function buildMarkdownHtml($message)
+    {
+        return fn ($data) => $this->markdownRenderer($message)->render(
+            $message->markdown, array_merge($data, $message->data()),
+        );
+    }
+
+    /**
+     * Build the text view for a Markdown message.
+     *
+     * @param  \Illuminate\Notifications\Messages\MailMessage  $message
+     * @return \Closure
+     */
+    protected function buildMarkdownText($message)
+    {
+        return fn ($data) => $this->markdownRenderer($message)->renderText(
+            $message->markdown, array_merge($data, $message->data()),
+        );
+    }
+
+    /**
+     * Get the Markdown implementation.
+     *
+     * @param  \Illuminate\Notifications\Messages\MailMessage  $message
+     * @return \Illuminate\Mail\Markdown
+     */
+    protected function markdownRenderer($message)
+    {
+        $config = Container::getInstance()->get(ConfigRepository::class);
+
+        $theme = $message->theme ?? $config->get('mail.markdown.theme', 'default');
+
+        return $this->markdown->theme($theme);
+    }
+
     /**
      * Get additional meta-data to pass along with the view data.
      *
@@ -141,7 +183,19 @@ class MailChannel
         $this->addAttachments($mailMessage, $message);
 
         if (! is_null($message->priority)) {
-            $mailMessage->setPriority($message->priority);
+            $mailMessage->priority($message->priority);
+        }
+
+        if ($message->tags) {
+            foreach ($message->tags as $tag) {
+                $mailMessage->getHeaders()->add(new TagHeader($tag));
+            }
+        }
+
+        if ($message->metadata) {
+            foreach ($message->metadata as $key => $value) {
+                $mailMessage->getHeaders()->add(new MetadataHeader($key, $value));
+            }
         }
 
         $this->runCallbacks($mailMessage, $message);
@@ -209,7 +263,7 @@ class MailChannel
             $recipients = [$recipients];
         }
 
-        return collect($recipients)->mapWithKeys(function ($recipient, $email) {
+        return (new Collection($recipients))->mapWithKeys(function ($recipient, $email) {
             return is_numeric($email)
                     ? [$email => (is_string($recipient) ? $recipient : $recipient->email)]
                     : [$email => $recipient];
@@ -244,7 +298,7 @@ class MailChannel
     protected function runCallbacks($mailMessage, $message)
     {
         foreach ($message->callbacks as $callback) {
-            $callback($mailMessage->getSwiftMessage());
+            $callback($mailMessage->getSymfonyMessage());
         }
 
         return $this;
