@@ -8,9 +8,12 @@ use Illuminate\Auth\Access\HandlesAuthorization;
 use Illuminate\Auth\Access\Response;
 use Illuminate\Container\Container;
 use InvalidArgumentException;
+use PHPUnit\Framework\Attributes\DataProvider;
 use PHPUnit\Framework\TestCase;
 use stdClass;
 
+include_once 'Enums.php';
+
 class AuthAccessGateTest extends TestCase
 {
     public function testBasicClosuresCanBeDefined()
@@ -325,6 +328,39 @@ class AuthAccessGateTest extends TestCase
         $this->assertTrue($gate->denies('deny'));
     }
 
+    public function testCanDefineGatesUsingBackedEnum()
+    {
+        $gate = $this->getBasicGate();
+
+        $gate->define(AbilitiesEnum::VIEW_DASHBOARD, function ($user) {
+            return true;
+        });
+
+        $this->assertTrue($gate->allows('view-dashboard'));
+    }
+
+    public function testBackedEnumInAllows()
+    {
+        $gate = $this->getBasicGate();
+
+        $gate->define(AbilitiesEnum::VIEW_DASHBOARD, function ($user) {
+            return true;
+        });
+
+        $this->assertTrue($gate->allows(AbilitiesEnum::VIEW_DASHBOARD));
+    }
+
+    public function testBackedEnumInDenies()
+    {
+        $gate = $this->getBasicGate();
+
+        $gate->define(AbilitiesEnum::VIEW_DASHBOARD, function ($user) {
+            return false;
+        });
+
+        $this->assertTrue($gate->denies(AbilitiesEnum::VIEW_DASHBOARD));
+    }
+
     public function testArrayAbilitiesInAllows()
     {
         $gate = $this->getBasicGate();
@@ -335,12 +371,15 @@ class AuthAccessGateTest extends TestCase
         $gate->define('allow_2', function ($user) {
             return true;
         });
+        $gate->define(AbilitiesEnum::VIEW_DASHBOARD, function ($user) {
+            return true;
+        });
         $gate->define('deny', function ($user) {
             return false;
         });
 
         $this->assertTrue($gate->allows(['allow_1']));
-        $this->assertTrue($gate->allows(['allow_1', 'allow_2']));
+        $this->assertTrue($gate->allows(['allow_1', 'allow_2', AbilitiesEnum::VIEW_DASHBOARD]));
         $this->assertFalse($gate->allows(['allow_1', 'allow_2', 'deny']));
         $this->assertFalse($gate->allows(['deny', 'allow_1', 'allow_2']));
     }
@@ -355,12 +394,15 @@ class AuthAccessGateTest extends TestCase
         $gate->define('deny_2', function ($user) {
             return false;
         });
+        $gate->define(AbilitiesEnum::VIEW_DASHBOARD, function ($user) {
+            return false;
+        });
         $gate->define('allow', function ($user) {
             return true;
         });
 
         $this->assertTrue($gate->denies(['deny_1']));
-        $this->assertTrue($gate->denies(['deny_1', 'deny_2']));
+        $this->assertTrue($gate->denies(['deny_1', 'deny_2', AbilitiesEnum::VIEW_DASHBOARD]));
         $this->assertTrue($gate->denies(['deny_1', 'allow']));
         $this->assertTrue($gate->denies(['allow', 'deny_1']));
         $this->assertFalse($gate->denies(['allow']));
@@ -596,9 +638,7 @@ class AuthAccessGateTest extends TestCase
         $this->assertSame(3, $counter);
     }
 
-    /**
-     * @dataProvider notCallableDataProvider
-     */
+    #[DataProvider('notCallableDataProvider')]
     public function testDefineSecondParameterShouldBeStringOrCallable($callback)
     {
         $this->expectException(InvalidArgumentException::class);
@@ -1076,13 +1116,39 @@ class AuthAccessGateTest extends TestCase
         $this->assertFalse($gate->check(['edit', 'update'], new AccessGateTestDummy));
     }
 
+    public function testAnyAbilitiesCheckUsingBackedEnum()
+    {
+        $gate = $this->getBasicGate();
+
+        $gate->policy(AccessGateTestDummy::class, AccessGateTestPolicyWithAllPermissions::class);
+
+        $this->assertTrue($gate->any(['edit', AbilitiesEnum::UPDATE], new AccessGateTestDummy));
+    }
+
+    public function testNoneAbilitiesCheckUsingBackedEnum()
+    {
+        $gate = $this->getBasicGate();
+
+        $gate->policy(AccessGateTestDummy::class, AccessGateTestPolicyWithNoPermissions::class);
+
+        $this->assertTrue($gate->none(['edit', AbilitiesEnum::UPDATE], new AccessGateTestDummy));
+    }
+
+    public function testAbilitiesCheckUsingBackedEnum()
+    {
+        $gate = $this->getBasicGate();
+
+        $gate->policy(AccessGateTestDummy::class, AccessGateTestPolicyWithAllPermissions::class);
+
+        $this->assertTrue($gate->check(['edit', AbilitiesEnum::UPDATE], new AccessGateTestDummy));
+    }
+
     /**
-     * @dataProvider hasAbilitiesTestDataProvider
-     *
      * @param  array  $abilitiesToSet
      * @param  array|string  $abilitiesToCheck
      * @param  bool  $expectedHasValue
      */
+    #[DataProvider('hasAbilitiesTestDataProvider')]
     public function testHasAbilities($abilitiesToSet, $abilitiesToCheck, $expectedHasValue)
     {
         $gate = $this->getBasicGate();
