@@ -9,6 +9,13 @@ use stdClass;
 
 class CacheArrayStoreTest extends TestCase
 {
+    protected function tearDown(): void
+    {
+        parent::tearDown();
+
+        Carbon::setTestNow(null);
+    }
+
     public function testItemsCanBeSetAndRetrieved()
     {
         $store = new ArrayStore;
@@ -17,6 +24,20 @@ class CacheArrayStoreTest extends TestCase
         $this->assertSame('bar', $store->get('foo'));
     }
 
+    public function testCacheTtl(): void
+    {
+        $store = new ArrayStore();
+
+        Carbon::setTestNow('2000-01-01 00:00:00.500'); // 500 milliseconds past
+        $store->put('hello', 'world', 1);
+
+        Carbon::setTestNow('2000-01-01 00:00:01.499'); // progress 0.999 seconds
+        $this->assertSame('world', $store->get('hello'));
+
+        Carbon::setTestNow('2000-01-01 00:00:01.500'); // progress 0.001 seconds. 1 second since putting into cache.
+        $this->assertNull($store->get('hello'));
+    }
+
     public function testMultipleItemsCanBeSetAndRetrieved()
     {
         $store = new ArrayStore;
@@ -38,6 +59,7 @@ class CacheArrayStoreTest extends TestCase
     public function testItemsCanExpire()
     {
         Carbon::setTestNow(Carbon::now());
+
         $store = new ArrayStore;
 
         $store->put('foo', 'bar', 10);
@@ -45,7 +67,6 @@ class CacheArrayStoreTest extends TestCase
         $result = $store->get('foo');
 
         $this->assertNull($result);
-        Carbon::setTestNow(null);
     }
 
     public function testStoreItemForeverProperlyStoresInArray()
@@ -65,6 +86,33 @@ class CacheArrayStoreTest extends TestCase
         $result = $store->increment('foo');
         $this->assertEquals(2, $result);
         $this->assertEquals(2, $store->get('foo'));
+
+        $result = $store->increment('foo', 2);
+        $this->assertEquals(4, $result);
+        $this->assertEquals(4, $store->get('foo'));
+    }
+
+    public function testValuesGetCastedByIncrementOrDecrement()
+    {
+        $store = new ArrayStore;
+        $store->put('foo', '1', 10);
+        $result = $store->increment('foo');
+        $this->assertEquals(2, $result);
+        $this->assertEquals(2, $store->get('foo'));
+
+        $store->put('bar', '1', 10);
+        $result = $store->decrement('bar');
+        $this->assertEquals(0, $result);
+        $this->assertEquals(0, $store->get('bar'));
+    }
+
+    public function testIncrementNonNumericValues()
+    {
+        $store = new ArrayStore;
+        $store->put('foo', 'I am string', 10);
+        $result = $store->increment('foo');
+        $this->assertEquals(1, $result);
+        $this->assertEquals(1, $store->get('foo'));
     }
 
     public function testNonExistingKeysCanBeIncremented()
@@ -73,11 +121,16 @@ class CacheArrayStoreTest extends TestCase
         $result = $store->increment('foo');
         $this->assertEquals(1, $result);
         $this->assertEquals(1, $store->get('foo'));
+
+        // Will be there forever
+        Carbon::setTestNow(Carbon::now()->addYears(10));
+        $this->assertEquals(1, $store->get('foo'));
     }
 
     public function testExpiredKeysAreIncrementedLikeNonExistingKeys()
     {
         Carbon::setTestNow(Carbon::now());
+
         $store = new ArrayStore;
 
         $store->put('foo', 999, 10);
@@ -85,7 +138,6 @@ class CacheArrayStoreTest extends TestCase
         $result = $store->increment('foo');
 
         $this->assertEquals(1, $result);
-        Carbon::setTestNow(null);
     }
 
     public function testValuesCanBeDecremented()
@@ -95,6 +147,10 @@ class CacheArrayStoreTest extends TestCase
         $result = $store->decrement('foo');
         $this->assertEquals(0, $result);
         $this->assertEquals(0, $store->get('foo'));
+
+        $result = $store->decrement('foo', 2);
+        $this->assertEquals(-2, $result);
+        $this->assertEquals(-2, $store->get('foo'));
     }
 
     public function testItemsCanBeRemoved()
@@ -135,6 +191,7 @@ class CacheArrayStoreTest extends TestCase
     public function testCanAcquireLockAgainAfterExpiry()
     {
         Carbon::setTestNow(Carbon::now());
+
         $store = new ArrayStore;
         $lock = $store->lock('foo', 10);
         $lock->acquire();
@@ -146,6 +203,7 @@ class CacheArrayStoreTest extends TestCase
     public function testLockExpirationLowerBoundary()
     {
         Carbon::setTestNow(Carbon::now());
+
         $store = new ArrayStore;
         $lock = $store->lock('foo', 10);
         $lock->acquire();
@@ -156,7 +214,6 @@ class CacheArrayStoreTest extends TestCase
 
     public function testLockWithNoExpirationNeverExpires()
     {
-        Carbon::setTestNow(Carbon::now());
         $store = new ArrayStore;
         $lock = $store->lock('foo');
         $lock->acquire();
@@ -205,7 +262,10 @@ class CacheArrayStoreTest extends TestCase
         $store->put('object', $object, 10);
         $object->bar = true;
 
-        $this->assertObjectNotHasAttribute('bar', $store->get('object'));
+        $retrievedObject = $store->get('object');
+
+        $this->assertTrue($retrievedObject->foo);
+        $this->assertFalse(property_exists($retrievedObject, 'bar'));
     }
 
     public function testValuesAreStoredByReferenceIfSerializationIsDisabled()
@@ -217,7 +277,10 @@ class CacheArrayStoreTest extends TestCase
         $store->put('object', $object, 10);
         $object->bar = true;
 
-        $this->assertObjectHasAttribute('bar', $store->get('object'));
+        $retrievedObject = $store->get('object');
+
+        $this->assertTrue($retrievedObject->foo);
+        $this->assertTrue($retrievedObject->bar);
     }
 
     public function testReleasingLockAfterAlreadyForceReleasedByAnotherOwnerFails()
@@ -230,4 +293,36 @@ class CacheArrayStoreTest extends TestCase
 
         $this->assertFalse($wannabeOwner->release());
     }
+
+    public function testOwnerStatusCanBeCheckedAfterRestoringLock()
+    {
+        $store = new ArrayStore;
+        $firstLock = $store->lock('foo', 10);
+
+        $this->assertTrue($firstLock->get());
+        $owner = $firstLock->owner();
+
+        $secondLock = $store->restoreLock('foo', $owner);
+        $this->assertTrue($secondLock->isOwnedByCurrentProcess());
+    }
+
+    public function testOtherOwnerDoesNotOwnLockAfterRestore()
+    {
+        $store = new ArrayStore;
+        $firstLock = $store->lock('foo', 10);
+
+        $this->assertTrue($firstLock->get());
+
+        $secondLock = $store->restoreLock('foo', 'other_owner');
+
+        $this->assertFalse($secondLock->isOwnedByCurrentProcess());
+    }
+
+    public function testRestoringNonExistingLockDoesNotOwnAnything()
+    {
+        $store = new ArrayStore;
+        $firstLock = $store->restoreLock('foo', 'owner');
+
+        $this->assertFalse($firstLock->isOwnedByCurrentProcess());
+    }
 }
