@@ -2,6 +2,7 @@
 
 namespace Illuminate\Tests\Cache;
 
+use Illuminate\Cache\ArrayStore;
 use Illuminate\Cache\RateLimiter;
 use Illuminate\Contracts\Cache\Repository as Cache;
 use Mockery as m;
@@ -20,6 +21,7 @@ class CacheRateLimiterTest extends TestCase
         $cache->shouldReceive('get')->once()->with('key', 0)->andReturn(1);
         $cache->shouldReceive('has')->once()->with('key:timer')->andReturn(true);
         $cache->shouldReceive('add')->never();
+        $cache->shouldReceive('getStore')->andReturn(new ArrayStore);
         $rateLimiter = new RateLimiter($cache);
 
         $this->assertTrue($rateLimiter->tooManyAttempts('key', 1));
@@ -31,6 +33,7 @@ class CacheRateLimiterTest extends TestCase
         $cache->shouldReceive('add')->once()->with('key:timer', m::type('int'), 1)->andReturn(true);
         $cache->shouldReceive('add')->once()->with('key', 0, 1)->andReturn(true);
         $cache->shouldReceive('increment')->once()->with('key', 1)->andReturn(1);
+        $cache->shouldReceive('getStore')->andReturn(new ArrayStore);
         $rateLimiter = new RateLimiter($cache);
 
         $rateLimiter->hit('key', 1);
@@ -42,11 +45,24 @@ class CacheRateLimiterTest extends TestCase
         $cache->shouldReceive('add')->once()->with('key:timer', m::type('int'), 1)->andReturn(true);
         $cache->shouldReceive('add')->once()->with('key', 0, 1)->andReturn(true);
         $cache->shouldReceive('increment')->once()->with('key', 5)->andReturn(5);
+        $cache->shouldReceive('getStore')->andReturn(new ArrayStore);
         $rateLimiter = new RateLimiter($cache);
 
         $rateLimiter->increment('key', 1, 5);
     }
 
+    public function testDecrementProperlyDecrementsAttemptCount()
+    {
+        $cache = m::mock(Cache::class);
+        $cache->shouldReceive('add')->once()->with('key:timer', m::type('int'), 1)->andReturn(true);
+        $cache->shouldReceive('add')->once()->with('key', 0, 1)->andReturn(true);
+        $cache->shouldReceive('increment')->once()->with('key', -5)->andReturn(-5);
+        $cache->shouldReceive('getStore')->andReturn(new ArrayStore);
+        $rateLimiter = new RateLimiter($cache);
+
+        $rateLimiter->decrement('key', 1, 5);
+    }
+
     public function testHitHasNoMemoryLeak()
     {
         $cache = m::mock(Cache::class);
@@ -54,6 +70,7 @@ class CacheRateLimiterTest extends TestCase
         $cache->shouldReceive('add')->once()->with('key', 0, 1)->andReturn(false);
         $cache->shouldReceive('increment')->once()->with('key', 1)->andReturn(1);
         $cache->shouldReceive('put')->once()->with('key', 1, 1);
+        $cache->shouldReceive('getStore')->andReturn(new ArrayStore);
         $rateLimiter = new RateLimiter($cache);
 
         $rateLimiter->hit('key', 1);
@@ -63,6 +80,7 @@ class CacheRateLimiterTest extends TestCase
     {
         $cache = m::mock(Cache::class);
         $cache->shouldReceive('get')->once()->with('key', 0)->andReturn(3);
+        $cache->shouldReceive('getStore')->andReturn(new ArrayStore);
         $rateLimiter = new RateLimiter($cache);
 
         $this->assertEquals(2, $rateLimiter->retriesLeft('key', 5));
@@ -73,6 +91,7 @@ class CacheRateLimiterTest extends TestCase
         $cache = m::mock(Cache::class);
         $cache->shouldReceive('forget')->once()->with('key');
         $cache->shouldReceive('forget')->once()->with('key:timer');
+        $cache->shouldReceive('getStore')->andReturn(new ArrayStore);
         $rateLimiter = new RateLimiter($cache);
 
         $rateLimiter->clear('key');
@@ -82,6 +101,7 @@ class CacheRateLimiterTest extends TestCase
     {
         $cache = m::mock(Cache::class);
         $cache->shouldReceive('get')->andReturn(now()->subSeconds(60)->getTimestamp(), null);
+        $cache->shouldReceive('getStore')->andReturn(new ArrayStore);
         $rateLimiter = new RateLimiter($cache);
 
         $this->assertTrue($rateLimiter->availableIn('key:timer') >= 0);
@@ -95,6 +115,7 @@ class CacheRateLimiterTest extends TestCase
         $cache->shouldReceive('add')->once()->with('key:timer', m::type('int'), 1);
         $cache->shouldReceive('add')->once()->with('key', 0, 1)->andReturns(1);
         $cache->shouldReceive('increment')->once()->with('key', 1)->andReturn(1);
+        $cache->shouldReceive('getStore')->andReturn(new ArrayStore);
 
         $executed = false;
 
@@ -113,6 +134,7 @@ class CacheRateLimiterTest extends TestCase
         $cache->shouldReceive('add')->times(6)->with('key:timer', m::type('int'), 1);
         $cache->shouldReceive('add')->times(6)->with('key', 0, 1)->andReturns(1);
         $cache->shouldReceive('increment')->times(6)->with('key', 1)->andReturn(1);
+        $cache->shouldReceive('getStore')->andReturn(new ArrayStore);
 
         $rateLimiter = new RateLimiter($cache);
 
@@ -146,6 +168,7 @@ class CacheRateLimiterTest extends TestCase
         $cache = m::mock(Cache::class);
         $cache->shouldReceive('get')->once()->with('key', 0)->andReturn(2);
         $cache->shouldReceive('has')->once()->with('key:timer')->andReturn(true);
+        $cache->shouldReceive('getStore')->andReturn(new ArrayStore);
 
         $executed = false;
 
@@ -163,6 +186,7 @@ class CacheRateLimiterTest extends TestCase
         $cache->shouldReceive('get')->once()->with('john', 0)->andReturn(1);
         $cache->shouldReceive('has')->once()->with('john:timer')->andReturn(true);
         $cache->shouldReceive('add')->never();
+        $cache->shouldReceive('getStore')->andReturn(new ArrayStore);
         $rateLimiter = new RateLimiter($cache);
 
         $this->assertTrue($rateLimiter->tooManyAttempts('jôhn', 1));
@@ -179,6 +203,7 @@ class CacheRateLimiterTest extends TestCase
         $cache->shouldReceive('get')->once()->with($cleanedKey, 0)->andReturn(1);
         $cache->shouldReceive('has')->once()->with("$cleanedKey:timer")->andReturn(true);
         $cache->shouldReceive('add')->never();
+        $cache->shouldReceive('getStore')->andReturn(new ArrayStore);
 
         $this->assertTrue($rateLimiter->tooManyAttempts($key, 1));
     }
