@@ -23,6 +23,14 @@ class ContainerResolveNonInstantiableTest extends TestCase
         $this->assertCount(0, $parent->child->objects);
         $this->assertSame(42, $parent->i);
     }
+
+    public function testResolveVariadicPrimitive()
+    {
+        $container = new Container;
+        $parent = $container->make(VariadicPrimitive::class);
+
+        $this->assertSame($parent->params, []);
+    }
 }
 
 interface TestInterface
@@ -36,7 +44,7 @@ class ParentClass
      */
     public $i;
 
-    public function __construct(TestInterface $testObject = null, int $i = 0)
+    public function __construct(?TestInterface $testObject = null, int $i = 0)
     {
         $this->i = $i;
     }
@@ -73,3 +81,16 @@ class ChildClass
         $this->objects = $objects;
     }
 }
+
+class VariadicPrimitive
+{
+    /**
+     * @var array
+     */
+    public $params;
+
+    public function __construct(...$params)
+    {
+        $this->params = $params;
+    }
+}
