@@ -5,18 +5,15 @@ namespace Illuminate\Tests\Database;
 use Illuminate\Database\Eloquent\Builder;
 use Illuminate\Database\Eloquent\Model;
 use Illuminate\Database\Eloquent\Relations\MorphToMany;
+use Illuminate\Database\Query\Expression;
+use Illuminate\Database\Query\Grammars\Grammar;
+use Mockery\Adapter\Phpunit\MockeryTestCase as TestCase;
 use Mockery as m;
-use PHPUnit\Framework\TestCase;
 use stdClass;
 
 class DatabaseEloquentMorphToManyTest extends TestCase
 {
-    protected function tearDown(): void
-    {
-        m::close();
-    }
-
-    public function testEagerConstraintsAreProperlyAdded()
+    public function testEagerConstraintsAreProperlyAdded(): void
     {
         $relation = $this->getRelation();
         $relation->getParent()->shouldReceive('getKeyName')->andReturn('id');
@@ -30,20 +27,19 @@ class DatabaseEloquentMorphToManyTest extends TestCase
         $relation->addEagerConstraints([$model1, $model2]);
     }
 
-    public function testAttachInsertsPivotTableRecord()
+    public function testAttachInsertsPivotTableRecord(): void
     {
         $relation = $this->getMockBuilder(MorphToMany::class)->onlyMethods(['touchIfTouching'])->setConstructorArgs($this->getRelationArguments())->getMock();
         $query = m::mock(stdClass::class);
         $query->shouldReceive('from')->once()->with('taggables')->andReturn($query);
         $query->shouldReceive('insert')->once()->with([['taggable_id' => 1, 'taggable_type' => get_class($relation->getParent()), 'tag_id' => 2, 'foo' => 'bar']])->andReturn(true);
-        $relation->getQuery()->shouldReceive('getQuery')->andReturn($mockQueryBuilder = m::mock(stdClass::class));
-        $mockQueryBuilder->shouldReceive('newQuery')->once()->andReturn($query);
+        $relation->getQuery()->getQuery()->shouldReceive('newQuery')->once()->andReturn($query);
         $relation->expects($this->once())->method('touchIfTouching');
 
         $relation->attach(2, ['foo' => 'bar']);
     }
 
-    public function testDetachRemovesPivotTableRecord()
+    public function testDetachRemovesPivotTableRecord(): void
     {
         $relation = $this->getMockBuilder(MorphToMany::class)->onlyMethods(['touchIfTouching'])->setConstructorArgs($this->getRelationArguments())->getMock();
         $query = m::mock(stdClass::class);
@@ -52,14 +48,13 @@ class DatabaseEloquentMorphToManyTest extends TestCase
         $query->shouldReceive('where')->once()->with('taggable_type', get_class($relation->getParent()))->andReturn($query);
         $query->shouldReceive('whereIn')->once()->with('taggables.tag_id', [1, 2, 3]);
         $query->shouldReceive('delete')->once()->andReturn(true);
-        $relation->getQuery()->shouldReceive('getQuery')->andReturn($mockQueryBuilder = m::mock(stdClass::class));
-        $mockQueryBuilder->shouldReceive('newQuery')->once()->andReturn($query);
+        $relation->getQuery()->getQuery()->shouldReceive('newQuery')->once()->andReturn($query);
         $relation->expects($this->once())->method('touchIfTouching');
 
         $this->assertTrue($relation->detach([1, 2, 3]));
     }
 
-    public function testDetachMethodClearsAllPivotRecordsWhenNoIDsAreGiven()
+    public function testDetachMethodClearsAllPivotRecordsWhenNoIDsAreGiven(): void
     {
         $relation = $this->getMockBuilder(MorphToMany::class)->onlyMethods(['touchIfTouching'])->setConstructorArgs($this->getRelationArguments())->getMock();
         $query = m::mock(stdClass::class);
@@ -68,21 +63,45 @@ class DatabaseEloquentMorphToManyTest extends TestCase
         $query->shouldReceive('where')->once()->with('taggable_type', get_class($relation->getParent()))->andReturn($query);
         $query->shouldReceive('whereIn')->never();
         $query->shouldReceive('delete')->once()->andReturn(true);
-        $relation->getQuery()->shouldReceive('getQuery')->andReturn($mockQueryBuilder = m::mock(stdClass::class));
-        $mockQueryBuilder->shouldReceive('newQuery')->once()->andReturn($query);
+        $relation->getQuery()->getQuery()->shouldReceive('newQuery')->once()->andReturn($query);
         $relation->expects($this->once())->method('touchIfTouching');
 
         $this->assertTrue($relation->detach());
     }
 
-    public function getRelation()
+    public function testQueryExpressionCanBePassedToDifferentPivotQueryBuilderClauses(): void
+    {
+        $value = 'pivot_value';
+        $column = new Expression("CONCAT(foo, '_', bar)");
+        $relation = $this->getRelation();
+        /** @var Builder|m\MockInterface $builder */
+        $builder = $relation->getQuery();
+
+        $builder->shouldReceive('where')->with($column, '=', $value, 'and')->times(2)->andReturnSelf();
+        $relation->wherePivot($column, '=', $value);
+        $relation->withPivotValue($column, $value);
+
+        $builder->shouldReceive('whereBetween')->with($column, [$value, $value], 'and', false)->once()->andReturnSelf();
+        $relation->wherePivotBetween($column, [$value, $value]);
+
+        $builder->shouldReceive('whereIn')->with($column, [$value], 'and', false)->once()->andReturnSelf();
+        $relation->wherePivotIn($column, [$value]);
+
+        $builder->shouldReceive('whereNull')->with($column, 'and', false)->once()->andReturnSelf();
+        $relation->wherePivotNull($column);
+
+        $builder->shouldReceive('orderBy')->with($column, 'asc')->once()->andReturnSelf();
+        $relation->orderByPivot($column);
+    }
+
+    public function getRelation(): MorphToMany
     {
         [$builder, $parent] = $this->getRelationArguments();
 
         return new MorphToMany($builder, $parent, 'taggable', 'taggables', 'taggable_id', 'tag_id', 'id', 'id');
     }
 
-    public function getRelationArguments()
+    public function getRelationArguments(): array
     {
         $parent = m::mock(Model::class);
         $parent->shouldReceive('getMorphClass')->andReturn(get_class($parent));
@@ -105,6 +124,13 @@ class DatabaseEloquentMorphToManyTest extends TestCase
         $builder->shouldReceive('where')->once()->with('taggables.taggable_id', '=', 1);
         $builder->shouldReceive('where')->once()->with('taggables.taggable_type', get_class($parent));
 
+        $grammar = m::mock(Grammar::class);
+        $grammar->shouldReceive('isExpression')->with(m::type(Expression::class))->andReturnTrue();
+        $grammar->shouldReceive('isExpression')->with(m::type('string'))->andReturnFalse();
+        $builder->shouldReceive('getQuery')->andReturn(
+            m::mock(stdClass::class, ['getGrammar' => $grammar])
+        );
+
         return [$builder, $parent, 'taggable', 'taggables', 'taggable_id', 'tag_id', 'id', 'id', 'relation_name', false];
     }
 }
