@@ -4,6 +4,7 @@ namespace Illuminate\Tests\Database;
 
 use Illuminate\Database\ClassMorphViolationException;
 use Illuminate\Database\Eloquent\Model;
+use Illuminate\Database\Eloquent\Relations\Pivot;
 use Illuminate\Database\Eloquent\Relations\Relation;
 use PHPUnit\Framework\TestCase;
 
@@ -20,26 +21,26 @@ class DatabaseEloquentStrictMorphsTest extends TestCase
     {
         $this->expectException(ClassMorphViolationException::class);
 
-        $model = TestModel::make();
+        $model = new TestModel;
 
         $model->getMorphClass();
     }
 
     public function testStrictModeDoesNotThrowExceptionWhenMorphMap()
     {
-        $model = TestModel::make();
+        $model = new TestModel;
 
         Relation::morphMap([
             'test' => TestModel::class,
         ]);
 
         $morphName = $model->getMorphClass();
-        $this->assertEquals('test', $morphName);
+        $this->assertSame('test', $morphName);
     }
 
     public function testMapsCanBeEnforcedInOneMethod()
     {
-        $model = TestModel::make();
+        $model = new TestModel;
 
         Relation::requireMorphMap(false);
 
@@ -48,7 +49,23 @@ class DatabaseEloquentStrictMorphsTest extends TestCase
         ]);
 
         $morphName = $model->getMorphClass();
-        $this->assertEquals('test', $morphName);
+        $this->assertSame('test', $morphName);
+    }
+
+    public function testMapIgnoreGenericPivotClass()
+    {
+        $pivotModel = new Pivot();
+
+        $pivotModel->getMorphClass();
+    }
+
+    public function testMapCanBeEnforcedToCustomPivotClass()
+    {
+        $this->expectException(ClassMorphViolationException::class);
+
+        $pivotModel = new TestPivotModel();
+
+        $pivotModel->getMorphClass();
     }
 
     protected function tearDown(): void
@@ -63,3 +80,7 @@ class DatabaseEloquentStrictMorphsTest extends TestCase
 class TestModel extends Model
 {
 }
+
+class TestPivotModel extends Pivot
+{
+}
