@@ -5,10 +5,12 @@ namespace Illuminate\Tests\Database;
 use Illuminate\Database\Connection;
 use Illuminate\Database\Schema\Blueprint;
 use Illuminate\Database\Schema\Builder;
+use Illuminate\Database\Schema\Grammars\MariaDbGrammar;
 use Illuminate\Database\Schema\Grammars\MySqlGrammar;
 use Illuminate\Database\Schema\Grammars\PostgresGrammar;
 use Illuminate\Database\Schema\Grammars\SQLiteGrammar;
 use Illuminate\Database\Schema\Grammars\SqlServerGrammar;
+use Illuminate\Tests\Database\Fixtures\Models\User;
 use Mockery as m;
 use PHPUnit\Framework\TestCase;
 
@@ -119,6 +121,7 @@ class DatabaseSchemaBlueprintTest extends TestCase
         $this->assertEquals(['alter table "users" add column "created" timestamp(0) without time zone not null default CURRENT_TIMESTAMP'], $blueprint->toSql($connection, new PostgresGrammar));
 
         $blueprint = clone $base;
+        $connection->shouldReceive('getServerVersion')->andReturn('3.35');
         $this->assertEquals(['alter table "users" add column "created" datetime not null default CURRENT_TIMESTAMP'], $blueprint->toSql($connection, new SQLiteGrammar));
 
         $blueprint = clone $base;
@@ -140,24 +143,13 @@ class DatabaseSchemaBlueprintTest extends TestCase
         $this->assertEquals(['alter table "users" add column "created" timestamp(0) without time zone not null default CURRENT_TIMESTAMP'], $blueprint->toSql($connection, new PostgresGrammar));
 
         $blueprint = clone $base;
+        $connection->shouldReceive('getServerVersion')->andReturn('3.35');
         $this->assertEquals(['alter table "users" add column "created" datetime not null default CURRENT_TIMESTAMP'], $blueprint->toSql($connection, new SQLiteGrammar));
 
         $blueprint = clone $base;
         $this->assertEquals(['alter table "users" add "created" datetime not null default CURRENT_TIMESTAMP'], $blueprint->toSql($connection, new SqlServerGrammar));
     }
 
-    public function testUnsignedDecimalTable()
-    {
-        $base = new Blueprint('users', function ($table) {
-            $table->unsignedDecimal('money', 10, 2)->useCurrent();
-        });
-
-        $connection = m::mock(Connection::class);
-
-        $blueprint = clone $base;
-        $this->assertEquals(['alter table `users` add `money` decimal(10, 2) unsigned not null'], $blueprint->toSql($connection, new MySqlGrammar));
-    }
-
     public function testRemoveColumn()
     {
         $base = new Blueprint('users', function ($table) {
@@ -173,14 +165,15 @@ class DatabaseSchemaBlueprintTest extends TestCase
         $this->assertEquals(['alter table `users` add `foo` varchar(255) not null'], $blueprint->toSql($connection, new MySqlGrammar));
     }
 
-    public function testRenameColumnWithoutDoctrine()
+    public function testRenameColumn()
     {
         $base = new Blueprint('users', function ($table) {
             $table->renameColumn('foo', 'bar');
         });
 
         $connection = m::mock(Connection::class);
-        $connection->shouldReceive('usingNativeSchemaOperations')->andReturn(true);
+        $connection->shouldReceive('getServerVersion')->andReturn('8.0.4');
+        $connection->shouldReceive('isMaria')->andReturn(false);
 
         $blueprint = clone $base;
         $this->assertEquals(['alter table `users` rename column `foo` to `bar`'], $blueprint->toSql($connection, new MySqlGrammar));
@@ -192,17 +185,67 @@ class DatabaseSchemaBlueprintTest extends TestCase
         $this->assertEquals(['alter table "users" rename column "foo" to "bar"'], $blueprint->toSql($connection, new SQLiteGrammar));
 
         $blueprint = clone $base;
-        $this->assertEquals(['sp_rename \'"users"."foo"\', "bar", \'COLUMN\''], $blueprint->toSql($connection, new SqlServerGrammar));
+        $this->assertEquals(['sp_rename N\'"users"."foo"\', "bar", N\'COLUMN\''], $blueprint->toSql($connection, new SqlServerGrammar));
     }
 
-    public function testDropColumnWithoutDoctrine()
+    public function testNativeRenameColumnOnMysql57()
+    {
+        $blueprint = new Blueprint('users', function ($table) {
+            $table->renameColumn('name', 'title');
+            $table->renameColumn('id', 'key');
+            $table->renameColumn('generated', 'new_generated');
+        });
+
+        $connection = m::mock(Connection::class);
+        $connection->shouldReceive('isMaria')->andReturn(false);
+        $connection->shouldReceive('getServerVersion')->andReturn('5.7');
+        $connection->shouldReceive('getSchemaBuilder->getColumns')->andReturn([
+            ['name' => 'name', 'type' => 'varchar(255)', 'type_name' => 'varchar', 'nullable' => true, 'collation' => 'utf8mb4_unicode_ci', 'default' => 'foo', 'comment' => null, 'auto_increment' => false, 'generation' => null],
+            ['name' => 'id', 'type' => 'bigint unsigned', 'type_name' => 'bigint', 'nullable' => false, 'collation' => null, 'default' => null, 'comment' => 'lorem ipsum', 'auto_increment' => true, 'generation' => null],
+            ['name' => 'generated', 'type' => 'int', 'type_name' => 'int', 'nullable' => false, 'collation' => null, 'default' => null, 'comment' => null, 'auto_increment' => false, 'generation' => ['type' => 'stored', 'expression' => 'expression']],
+        ]);
+
+        $this->assertEquals([
+            "alter table `users` change `name` `title` varchar(255) collate 'utf8mb4_unicode_ci' null default 'foo'",
+            "alter table `users` change `id` `key` bigint unsigned not null auto_increment comment 'lorem ipsum'",
+            'alter table `users` change `generated` `new_generated` int as (expression) stored not null',
+        ], $blueprint->toSql($connection, new MySqlGrammar));
+    }
+
+    public function testNativeRenameColumnOnLegacyMariaDB()
+    {
+        $blueprint = new Blueprint('users', function ($table) {
+            $table->renameColumn('name', 'title');
+            $table->renameColumn('id', 'key');
+            $table->renameColumn('generated', 'new_generated');
+            $table->renameColumn('foo', 'bar');
+        });
+
+        $connection = m::mock(Connection::class);
+        $connection->shouldReceive('isMaria')->andReturn(true);
+        $connection->shouldReceive('getServerVersion')->andReturn('10.1.35');
+        $connection->shouldReceive('getSchemaBuilder->getColumns')->andReturn([
+            ['name' => 'name', 'type' => 'varchar(255)', 'type_name' => 'varchar', 'nullable' => true, 'collation' => 'utf8mb4_unicode_ci', 'default' => 'foo', 'comment' => null, 'auto_increment' => false, 'generation' => null],
+            ['name' => 'id', 'type' => 'bigint unsigned', 'type_name' => 'bigint', 'nullable' => false, 'collation' => null, 'default' => null, 'comment' => 'lorem ipsum', 'auto_increment' => true, 'generation' => null],
+            ['name' => 'generated', 'type' => 'int', 'type_name' => 'int', 'nullable' => false, 'collation' => null, 'default' => null, 'comment' => null, 'auto_increment' => false, 'generation' => ['type' => 'stored', 'expression' => 'expression']],
+            ['name' => 'foo', 'type' => 'int', 'type_name' => 'int', 'nullable' => true, 'collation' => null, 'default' => 'NULL', 'comment' => null, 'auto_increment' => false, 'generation' => null],
+        ]);
+
+        $this->assertEquals([
+            "alter table `users` change `name` `title` varchar(255) collate 'utf8mb4_unicode_ci' null default 'foo'",
+            "alter table `users` change `id` `key` bigint unsigned not null auto_increment comment 'lorem ipsum'",
+            'alter table `users` change `generated` `new_generated` int as (expression) stored not null',
+            'alter table `users` change `foo` `bar` int null default NULL',
+        ], $blueprint->toSql($connection, new MariaDbGrammar));
+    }
+
+    public function testDropColumn()
     {
         $base = new Blueprint('users', function ($table) {
             $table->dropColumn('foo');
         });
 
         $connection = m::mock(Connection::class);
-        $connection->shouldReceive('usingNativeSchemaOperations')->andReturn(true);
 
         $blueprint = clone $base;
         $this->assertEquals(['alter table `users` drop `foo`'], $blueprint->toSql($connection, new MySqlGrammar));
@@ -211,6 +254,7 @@ class DatabaseSchemaBlueprintTest extends TestCase
         $this->assertEquals(['alter table "users" drop column "foo"'], $blueprint->toSql($connection, new PostgresGrammar));
 
         $blueprint = clone $base;
+        $connection->shouldReceive('getServerVersion')->andReturn('3.35');
         $this->assertEquals(['alter table "users" drop column "foo"'], $blueprint->toSql($connection, new SQLiteGrammar));
 
         $blueprint = clone $base;
@@ -247,7 +291,8 @@ class DatabaseSchemaBlueprintTest extends TestCase
         $blueprint = clone $base;
 
         $this->assertEquals([
-            'alter table `comments` add `commentable_type` varchar(255) not null, add `commentable_id` bigint unsigned not null',
+            'alter table `comments` add `commentable_type` varchar(255) not null',
+            'alter table `comments` add `commentable_id` bigint unsigned not null',
             'alter table `comments` add index `comments_commentable_type_commentable_id_index`(`commentable_type`, `commentable_id`)',
         ], $blueprint->toSql($connection, new MySqlGrammar));
     }
@@ -263,7 +308,8 @@ class DatabaseSchemaBlueprintTest extends TestCase
         $blueprint = clone $base;
 
         $this->assertEquals([
-            'alter table `comments` add `commentable_type` varchar(255) null, add `commentable_id` bigint unsigned null',
+            'alter table `comments` add `commentable_type` varchar(255) null',
+            'alter table `comments` add `commentable_id` bigint unsigned null',
             'alter table `comments` add index `comments_commentable_type_commentable_id_index`(`commentable_type`, `commentable_id`)',
         ], $blueprint->toSql($connection, new MySqlGrammar));
     }
@@ -281,7 +327,8 @@ class DatabaseSchemaBlueprintTest extends TestCase
         $blueprint = clone $base;
 
         $this->assertEquals([
-            'alter table `comments` add `commentable_type` varchar(255) not null, add `commentable_id` char(36) not null',
+            'alter table `comments` add `commentable_type` varchar(255) not null',
+            'alter table `comments` add `commentable_id` char(36) not null',
             'alter table `comments` add index `comments_commentable_type_commentable_id_index`(`commentable_type`, `commentable_id`)',
         ], $blueprint->toSql($connection, new MySqlGrammar));
     }
@@ -299,7 +346,8 @@ class DatabaseSchemaBlueprintTest extends TestCase
         $blueprint = clone $base;
 
         $this->assertEquals([
-            'alter table `comments` add `commentable_type` varchar(255) null, add `commentable_id` char(36) null',
+            'alter table `comments` add `commentable_type` varchar(255) null',
+            'alter table `comments` add `commentable_id` char(36) null',
             'alter table `comments` add index `comments_commentable_type_commentable_id_index`(`commentable_type`, `commentable_id`)',
         ], $blueprint->toSql($connection, new MySqlGrammar));
     }
@@ -317,7 +365,8 @@ class DatabaseSchemaBlueprintTest extends TestCase
         $blueprint = clone $base;
 
         $this->assertEquals([
-            'alter table `comments` add `commentable_type` varchar(255) not null, add `commentable_id` char(26) not null',
+            'alter table `comments` add `commentable_type` varchar(255) not null',
+            'alter table `comments` add `commentable_id` char(26) not null',
             'alter table `comments` add index `comments_commentable_type_commentable_id_index`(`commentable_type`, `commentable_id`)',
         ], $blueprint->toSql($connection, new MySqlGrammar));
     }
@@ -335,7 +384,8 @@ class DatabaseSchemaBlueprintTest extends TestCase
         $blueprint = clone $base;
 
         $this->assertEquals([
-            'alter table `comments` add `commentable_type` varchar(255) null, add `commentable_id` char(26) null',
+            'alter table `comments` add `commentable_type` varchar(255) null',
+            'alter table `comments` add `commentable_id` char(26) null',
             'alter table `comments` add index `comments_commentable_type_commentable_id_index`(`commentable_type`, `commentable_id`)',
         ], $blueprint->toSql($connection, new MySqlGrammar));
     }
@@ -355,12 +405,10 @@ class DatabaseSchemaBlueprintTest extends TestCase
         ], $blueprint->toSql($connection, new MySqlGrammar));
     }
 
-    public function testGenerateRelationshipColumnWithUuidModel()
+    public function testGenerateRelationshipColumnWithNonIncrementalModel()
     {
-        require_once __DIR__.'/stubs/EloquentModelUuidStub.php';
-
         $base = new Blueprint('posts', function ($table) {
-            $table->foreignIdFor('EloquentModelUuidStub');
+            $table->foreignIdFor(Fixtures\Models\EloquentModelUsingNonIncrementedInt::class);
         });
 
         $connection = m::mock(Connection::class);
@@ -368,16 +416,29 @@ class DatabaseSchemaBlueprintTest extends TestCase
         $blueprint = clone $base;
 
         $this->assertEquals([
-            'alter table `posts` add `eloquent_model_uuid_stub_id` char(36) not null',
+            'alter table `posts` add `model_using_non_incremented_int_id` bigint unsigned not null',
+        ], $blueprint->toSql($connection, new MySqlGrammar));
+    }
+
+    public function testGenerateRelationshipColumnWithUuidModel()
+    {
+        $base = new Blueprint('posts', function ($table) {
+            $table->foreignIdFor(Fixtures\Models\EloquentModelUsingUuid::class);
+        });
+
+        $connection = m::mock(Connection::class);
+
+        $blueprint = clone $base;
+
+        $this->assertEquals([
+            'alter table `posts` add `model_using_uuid_id` char(36) not null',
         ], $blueprint->toSql($connection, new MySqlGrammar));
     }
 
     public function testGenerateRelationshipColumnWithUlidModel()
     {
-        require_once __DIR__.'/stubs/EloquentModelUlidStub.php';
-
         $base = new Blueprint('posts', function (Blueprint $table) {
-            $table->foreignIdFor('EloquentModelUlidStub');
+            $table->foreignIdFor(Fixtures\Models\EloquentModelUsingUlid::class);
         });
 
         $connection = m::mock(Connection::class);
@@ -385,16 +446,48 @@ class DatabaseSchemaBlueprintTest extends TestCase
         $blueprint = clone $base;
 
         $this->assertEquals([
-            'alter table "posts" add column "eloquent_model_ulid_stub_id" char(26) not null',
+            'alter table "posts" add column "model_using_ulid_id" char(26) not null',
         ], $blueprint->toSql($connection, new PostgresGrammar));
 
         $blueprint = clone $base;
 
         $this->assertEquals([
-            'alter table `posts` add `eloquent_model_ulid_stub_id` char(26) not null',
+            'alter table `posts` add `model_using_ulid_id` char(26) not null',
         ], $blueprint->toSql($connection, new MySqlGrammar()));
     }
 
+    public function testGenerateRelationshipConstrainedColumn()
+    {
+        $base = new Blueprint('posts', function ($table) {
+            $table->foreignIdFor('Illuminate\Foundation\Auth\User')->constrained();
+        });
+
+        $connection = m::mock(Connection::class);
+
+        $blueprint = clone $base;
+
+        $this->assertEquals([
+            'alter table `posts` add `user_id` bigint unsigned not null',
+            'alter table `posts` add constraint `posts_user_id_foreign` foreign key (`user_id`) references `users` (`id`)',
+        ], $blueprint->toSql($connection, new MySqlGrammar));
+    }
+
+    public function testGenerateRelationshipForModelWithNonStandardPrimaryKeyName()
+    {
+        $base = new Blueprint('posts', function ($table) {
+            $table->foreignIdFor(User::class)->constrained();
+        });
+
+        $connection = m::mock(Connection::class);
+
+        $blueprint = clone $base;
+
+        $this->assertEquals([
+            'alter table `posts` add `user_internal_id` bigint unsigned not null',
+            'alter table `posts` add constraint `posts_user_internal_id_foreign` foreign key (`user_internal_id`) references `users` (`internal_id`)',
+        ], $blueprint->toSql($connection, new MySqlGrammar));
+    }
+
     public function testDropRelationshipColumnWithIncrementalModel()
     {
         $base = new Blueprint('posts', function ($table) {
@@ -412,10 +505,8 @@ class DatabaseSchemaBlueprintTest extends TestCase
 
     public function testDropRelationshipColumnWithUuidModel()
     {
-        require_once __DIR__.'/stubs/EloquentModelUuidStub.php';
-
         $base = new Blueprint('posts', function ($table) {
-            $table->dropForeignIdFor('EloquentModelUuidStub');
+            $table->dropForeignIdFor(Fixtures\Models\EloquentModelUsingUuid::class);
         });
 
         $connection = m::mock(Connection::class);
@@ -423,7 +514,7 @@ class DatabaseSchemaBlueprintTest extends TestCase
         $blueprint = clone $base;
 
         $this->assertEquals([
-            'alter table `posts` drop foreign key `posts_eloquent_model_uuid_stub_id_foreign`',
+            'alter table `posts` drop foreign key `posts_model_using_uuid_id_foreign`',
         ], $blueprint->toSql($connection, new MySqlGrammar));
     }
 
@@ -445,10 +536,8 @@ class DatabaseSchemaBlueprintTest extends TestCase
 
     public function testDropConstrainedRelationshipColumnWithUuidModel()
     {
-        require_once __DIR__.'/stubs/EloquentModelUuidStub.php';
-
         $base = new Blueprint('posts', function ($table) {
-            $table->dropConstrainedForeignIdFor('EloquentModelUuidStub');
+            $table->dropConstrainedForeignIdFor(Fixtures\Models\EloquentModelUsingUuid::class);
         });
 
         $connection = m::mock(Connection::class);
@@ -456,8 +545,8 @@ class DatabaseSchemaBlueprintTest extends TestCase
         $blueprint = clone $base;
 
         $this->assertEquals([
-            'alter table `posts` drop foreign key `posts_eloquent_model_uuid_stub_id_foreign`',
-            'alter table `posts` drop `eloquent_model_uuid_stub_id`',
+            'alter table `posts` drop foreign key `posts_model_using_uuid_id_foreign`',
+            'alter table `posts` drop `model_using_uuid_id`',
         ], $blueprint->toSql($connection, new MySqlGrammar));
     }
 
@@ -475,6 +564,7 @@ class DatabaseSchemaBlueprintTest extends TestCase
         ], $blueprint->toSql($connection, new MySqlGrammar));
 
         $blueprint = clone $base;
+        $connection->shouldReceive('getServerVersion')->andReturn('3.35');
         $this->assertEquals([
             'alter table "posts" add column "note" text not null',
         ], $blueprint->toSql($connection, new SQLiteGrammar));
@@ -504,6 +594,7 @@ class DatabaseSchemaBlueprintTest extends TestCase
         ], $blueprint->toSql($connection, new MySqlGrammar));
 
         $blueprint = clone $base;
+        $connection->shouldReceive('getServerVersion')->andReturn('3.35');
         $this->assertEquals([
             'alter table "posts" add column "note" text',
         ], $blueprint->toSql($connection, new SQLiteGrammar));
@@ -519,6 +610,36 @@ class DatabaseSchemaBlueprintTest extends TestCase
         ], $blueprint->toSql($connection, new SqlServerGrammar));
     }
 
+    public function testRawColumn()
+    {
+        $base = new Blueprint('posts', function ($table) {
+            $table->rawColumn('legacy_boolean', 'INT(1)')->nullable();
+        });
+
+        $connection = m::mock(Connection::class);
+
+        $blueprint = clone $base;
+        $this->assertEquals([
+            'alter table `posts` add `legacy_boolean` INT(1) null',
+        ], $blueprint->toSql($connection, new MySqlGrammar));
+
+        $blueprint = clone $base;
+        $connection->shouldReceive('getServerVersion')->andReturn('3.35');
+        $this->assertEquals([
+            'alter table "posts" add column "legacy_boolean" INT(1)',
+        ], $blueprint->toSql($connection, new SQLiteGrammar));
+
+        $blueprint = clone $base;
+        $this->assertEquals([
+            'alter table "posts" add column "legacy_boolean" INT(1) null',
+        ], $blueprint->toSql($connection, new PostgresGrammar));
+
+        $blueprint = clone $base;
+        $this->assertEquals([
+            'alter table "posts" add "legacy_boolean" INT(1) null',
+        ], $blueprint->toSql($connection, new SqlServerGrammar));
+    }
+
     public function testTableComment()
     {
         $base = new Blueprint('posts', function (Blueprint $table) {
