@@ -4,6 +4,8 @@ namespace Illuminate\Tests\Filesystem;
 
 use Carbon\Carbon;
 use GuzzleHttp\Psr7\Stream;
+use Illuminate\Container\Container;
+use Illuminate\Contracts\Debug\ExceptionHandler;
 use Illuminate\Filesystem\FilesystemAdapter;
 use Illuminate\Filesystem\FilesystemManager;
 use Illuminate\Foundation\Application;
@@ -17,6 +19,7 @@ use League\Flysystem\UnableToReadFile;
 use League\Flysystem\UnableToRetrieveMetadata;
 use League\Flysystem\UnableToWriteFile;
 use Mockery as m;
+use PHPUnit\Framework\Attributes\RequiresPhpExtension;
 use PHPUnit\Framework\TestCase;
 use Symfony\Component\HttpFoundation\StreamedResponse;
 
@@ -426,9 +429,7 @@ class FilesystemAdapterTest extends TestCase
         $this->assertSame('normal file content', $filesystemAdapter->read($storagePath));
     }
 
-    /**
-     * @requires extension ftp
-     */
+    #[RequiresPhpExtension('ftp')]
     public function testCreateFtpDriver()
     {
         $filesystem = new FilesystemManager(new Application);
@@ -546,6 +547,126 @@ class FilesystemAdapterTest extends TestCase
         $this->fail('Exception was not thrown.');
     }
 
+    public function testReportExceptionsForGet()
+    {
+        $container = Container::getInstance();
+
+        $exceptionHandler = m::mock(ExceptionHandler::class);
+
+        $exceptionHandler->shouldReceive('report')
+            ->once()
+            ->andReturnUsing(function (UnableToReadFile $e) {
+                self::assertStringContainsString(
+                    'Unable to read file from location: foo.txt.',
+                    $e->getMessage(),
+                );
+            });
+
+        $container->bind(ExceptionHandler::class, function () use ($exceptionHandler) {
+            return $exceptionHandler;
+        });
+
+        $adapter = new FilesystemAdapter($this->filesystem, $this->adapter, ['report' => true]);
+
+        try {
+            $adapter->get('/foo.txt');
+        } catch (UnableToReadFile) {
+            $this->fail('Exception was thrown.');
+        }
+    }
+
+    public function testReportExceptionsForReadStream()
+    {
+        $container = Container::getInstance();
+
+        $exceptionHandler = m::mock(ExceptionHandler::class);
+
+        $exceptionHandler->shouldReceive('report')
+            ->once()
+            ->andReturnUsing(function (UnableToReadFile $e) {
+                self::assertStringContainsString(
+                    'Unable to read file from location: foo.txt.',
+                    $e->getMessage(),
+                );
+            });
+
+        $container->bind(ExceptionHandler::class, function () use ($exceptionHandler) {
+            return $exceptionHandler;
+        });
+
+        $adapter = new FilesystemAdapter($this->filesystem, $this->adapter, ['report' => true], $exceptionHandler);
+
+        try {
+            $adapter->readStream('/foo.txt');
+        } catch (UnableToReadFile) {
+            $this->fail('Exception was thrown.');
+        }
+    }
+
+    public function testReportExceptionsForPut()
+    {
+        $container = Container::getInstance();
+
+        $exceptionHandler = m::mock(ExceptionHandler::class);
+
+        $exceptionHandler->shouldReceive('report')
+            ->once()
+            ->andReturnUsing(function (UnableToWriteFile $e) {
+                self::assertStringContainsString(
+                    'Unable to write file at location: foo.txt.',
+                    $e->getMessage(),
+                );
+            });
+
+        $container->bind(ExceptionHandler::class, function () use ($exceptionHandler) {
+            return $exceptionHandler;
+        });
+
+        $this->filesystem->write('foo.txt', 'Hello World');
+
+        chmod(__DIR__.'/tmp/foo.txt', 0400);
+
+        $adapter = new FilesystemAdapter($this->filesystem, $this->adapter, ['report' => true], $exceptionHandler);
+
+        try {
+            $adapter->put('/foo.txt', 'Hello World!');
+        } catch (UnableToWriteFile) {
+            $this->fail('Exception was thrown.');
+        } finally {
+            chmod(__DIR__.'/tmp/foo.txt', 0600);
+        }
+    }
+
+    public function testReportExceptionsForMimeType()
+    {
+        $container = Container::getInstance();
+
+        $exceptionHandler = m::mock(ExceptionHandler::class);
+
+        $exceptionHandler->shouldReceive('report')
+            ->once()
+            ->andReturnUsing(function (UnableToRetrieveMetadata $e) {
+                self::assertStringContainsString(
+                    'Unable to retrieve the mime_type for file at location: unknown.mime-type.',
+                    $e->getMessage(),
+                );
+            });
+
+        $container->bind(ExceptionHandler::class, function () use ($exceptionHandler) {
+            return $exceptionHandler;
+        });
+
+        $this->filesystem->write('unknown.mime-type', '');
+
+        $adapter = new FilesystemAdapter($this->filesystem, $this->adapter, ['report' => true], $exceptionHandler);
+
+        try {
+            $adapter->mimeType('unknown.mime-type');
+        } catch (UnableToRetrieveMetadata) {
+            $this->fail('Exception was thrown.');
+        }
+    }
+
     public function testGetAllFiles()
     {
         $this->filesystem->write('body.txt', 'Hello World');
