@@ -166,6 +166,52 @@ class FoundationExceptionsHandlerTest extends TestCase
         $this->handler->report(new CustomException('Exception message'));
     }
 
+    public function testShouldReturnJson()
+    {
+        $this->request->shouldReceive('expectsJson')->once()->andReturn(true);
+        $e = new Exception('My custom error message');
+
+        $request = $this->request;
+
+        $shouldReturnJson = (fn () => $this->shouldReturnJson($request, $e))->call($this->handler);
+        $this->assertTrue($shouldReturnJson);
+
+        $this->request->shouldReceive('expectsJson')->once()->andReturn(false);
+
+        $shouldReturnJson = (fn () => $this->shouldReturnJson($request, $e))->call($this->handler);
+        $this->assertFalse($shouldReturnJson);
+    }
+
+    public function testShouldReturnJsonWhen()
+    {
+        $this->request->shouldReceive('expectsJson')->never();
+        $exception = new Exception('My custom error message');
+
+        $request = $this->request;
+
+        $this->handler->shouldRenderJsonWhen(function ($r, $e) use ($request, $exception) {
+            $this->assertSame($request, $r);
+            $this->assertSame($exception, $e);
+
+            return true;
+        });
+
+        $shouldReturnJson = (fn () => $this->shouldReturnJson($request, $exception))->call($this->handler);
+        $this->assertTrue($shouldReturnJson);
+
+        $this->handler->shouldRenderJsonWhen(function ($r, $e) use ($request, $exception) {
+            $this->assertSame($request, $r);
+            $this->assertSame($exception, $e);
+
+            return false;
+        });
+
+        $shouldReturnJson = (fn () => $this->shouldReturnJson($request, $exception))->call($this->handler);
+        $this->assertFalse($shouldReturnJson);
+
+        $this->assertSame(6, Assert::getCount());
+    }
+
     public function testReturnsJsonWithStackTraceWhenAjaxRequestAndDebugTrue()
     {
         $this->config->shouldReceive('get')->with('app.debug', null)->once()->andReturn(true);
@@ -313,15 +359,15 @@ class FoundationExceptionsHandlerTest extends TestCase
         $this->assertEquals($argumentExpected, $argumentActual);
     }
 
-    public function testSuspiciousOperationReturns404WithoutReporting()
+    public function testSuspiciousOperationReturns400WithoutReporting()
     {
         $this->config->shouldReceive('get')->with('app.debug', null)->once()->andReturn(true);
         $this->request->shouldReceive('expectsJson')->once()->andReturn(true);
 
         $response = $this->handler->render($this->request, new SuspiciousOperationException('Invalid method override "__CONSTRUCT"'));
 
-        $this->assertEquals(404, $response->getStatusCode());
-        $this->assertStringContainsString('"message": "Bad hostname provided."', $response->getContent());
+        $this->assertEquals(400, $response->getStatusCode());
+        $this->assertStringContainsString('"message": "Bad request."', $response->getContent());
 
         $logger = m::mock(LoggerInterface::class);
         $this->container->instance(LoggerInterface::class, $logger);
@@ -487,6 +533,42 @@ class FoundationExceptionsHandlerTest extends TestCase
         if ($testFailed) {
             Assert::fail('assertThrows failed: non matching message are thrown.');
         }
+
+        $this->assertThrows(function () {
+            throw new CustomException('Some message.');
+        }, function (CustomException $exception) {
+            return $exception->getMessage() === 'Some message.';
+        });
+
+        try {
+            $this->assertThrows(function () {
+                throw new CustomException('Some message.');
+            }, function (CustomException $exception) {
+                return false;
+            });
+            $testFailed = true;
+        } catch (AssertionFailedError) {
+            $testFailed = false;
+        }
+
+        if ($testFailed) {
+            Assert::fail('assertThrows failed: exception callback succeeded.');
+        }
+
+        try {
+            $this->assertThrows(function () {
+                throw new Exception('Some message.');
+            }, function (CustomException $exception) {
+                return true;
+            });
+            $testFailed = true;
+        } catch (AssertionFailedError) {
+            $testFailed = false;
+        }
+
+        if ($testFailed) {
+            Assert::fail('assertThrows failed: non matching exceptions are thrown.');
+        }
     }
 
     public function testItReportsDuplicateExceptions()
