@@ -1,4 +1,4 @@
-<<!DOCTYPE html>
+<!DOCTYPE html>
 <html>
 <head>
     <meta charset="utf-8">
@@ -6,4 +6,4 @@
     <title></title>
 </head>
 <body>
-<?php /**PATH /my-work-directory/resources/views/welcome.blade.php ENDPATH**/ ?>
\ No newline at end of file
+<?php /**PATH /my-work-directory/resources/views/welcome.blade.php ENDPATH**/ ?>
