@@ -25,8 +25,9 @@ class CacheTest extends TestCase
             'max_age=120',
             'no-transform',
             's_maxage=60',
+            'etag' => true,
         ]);
-        $this->assertSame('Illuminate\Http\Middleware\SetCacheHeaders:max_age=120;no-transform;s_maxage=60', $signature);
+        $this->assertSame('Illuminate\Http\Middleware\SetCacheHeaders:max_age=120;no-transform;s_maxage=60;etag', $signature);
 
         $signature = (string) Cache::using([
             'max_age' => 120,
@@ -172,4 +173,13 @@ class CacheTest extends TestCase
 
         $this->assertSame($time, $response->getLastModified()->getTimestamp());
     }
+
+    public function testItDoesNotSetEtagHeadersForBinaryContent()
+    {
+        $response = (new Cache)->handle(new Request, function () {
+            return new BinaryFileResponse(__DIR__.'/../fixtures/test.txt');
+        }, 'etag');
+
+        $this->assertNull($response->getEtag());
+    }
 }
