@@ -9,11 +9,12 @@ use Illuminate\Tests\Integration\Database\Fixtures\User;
 
 class EloquentCollectionFreshTest extends DatabaseTestCase
 {
-    protected function defineDatabaseMigrationsAfterDatabaseRefreshed()
+    protected function afterRefreshingDatabase()
     {
         Schema::create('users', function (Blueprint $table) {
             $table->increments('id');
             $table->string('email');
+            $table->timestamps();
         });
     }
 
