@@ -10,7 +10,7 @@ use Illuminate\Support\Str;
 
 class EloquentUpdateTest extends DatabaseTestCase
 {
-    protected function defineDatabaseMigrationsAfterDatabaseRefreshed()
+    protected function afterRefreshingDatabase()
     {
         Schema::create('test_model1', function (Blueprint $table) {
             $table->increments('id');
@@ -46,9 +46,12 @@ class EloquentUpdateTest extends DatabaseTestCase
         $this->assertCount(0, TestUpdateModel1::all());
     }
 
-    /** @group SkipMSSQL */
     public function testUpdateWithLimitsAndOrders()
     {
+        if ($this->driver === 'sqlsrv') {
+            $this->markTestSkipped('The limit keyword is not supported on MSSQL.');
+        }
+
         for ($i = 1; $i <= 10; $i++) {
             TestUpdateModel1::create();
         }
@@ -115,6 +118,24 @@ class EloquentUpdateTest extends DatabaseTestCase
         $this->assertEquals(1, $models[0]->counter);
         $this->assertEquals(0, $models[1]->counter);
     }
+
+    public function testIncrementOrDecrementIgnoresGlobalScopes()
+    {
+        /** @var TestUpdateModel3 $deletedModel */
+        $deletedModel = tap(TestUpdateModel3::create([
+            'counter' => 0,
+        ]), fn ($model) => $model->delete());
+
+        $deletedModel->increment('counter');
+
+        $this->assertEquals(1, $deletedModel->counter);
+
+        $deletedModel->fresh();
+        $this->assertEquals(1, $deletedModel->counter);
+
+        $deletedModel->decrement('counter');
+        $this->assertEquals(0, $deletedModel->fresh()->counter);
+    }
 }
 
 class TestUpdateModel1 extends Model
