@@ -3,13 +3,10 @@
 namespace Illuminate\Tests\Integration\Database\Postgres;
 
 use Illuminate\Tests\Integration\Database\DatabaseTestCase;
+use Orchestra\Testbench\Attributes\RequiresDatabase;
 
+#[RequiresDatabase('pgsql')]
 abstract class PostgresTestCase extends DatabaseTestCase
 {
-    protected function defineDatabaseMigrations()
-    {
-        if ($this->driver !== 'pgsql') {
-            $this->markTestSkipped('Test requires a PostgreSQL connection.');
-        }
-    }
+    //
 }
