@@ -3,13 +3,10 @@
 namespace Illuminate\Tests\Integration\Database\SqlServer;
 
 use Illuminate\Tests\Integration\Database\DatabaseTestCase;
+use Orchestra\Testbench\Attributes\RequiresDatabase;
 
+#[RequiresDatabase('sqlsrv')]
 abstract class SqlServerTestCase extends DatabaseTestCase
 {
-    protected function defineDatabaseMigrations()
-    {
-        if ($this->driver !== 'sqlsrv') {
-            $this->markTestSkipped('Test requires a SQL Server connection.');
-        }
-    }
+    //
 }
