@@ -13,7 +13,7 @@ class JsonSerializableResource implements JsonSerializable
         $this->resource = $resource;
     }
 
-    public function jsonSerialize()
+    public function jsonSerialize(): array
     {
         return [
             'id' => $this->resource->id,
