@@ -37,6 +37,8 @@ use Illuminate\Tests\Integration\Http\Fixtures\PostResourceWithOptionalPivotRela
 use Illuminate\Tests\Integration\Http\Fixtures\PostResourceWithOptionalRelationship;
 use Illuminate\Tests\Integration\Http\Fixtures\PostResourceWithOptionalRelationshipAggregates;
 use Illuminate\Tests\Integration\Http\Fixtures\PostResourceWithOptionalRelationshipCounts;
+use Illuminate\Tests\Integration\Http\Fixtures\PostResourceWithOptionalRelationshipExists;
+use Illuminate\Tests\Integration\Http\Fixtures\PostResourceWithOptionalRelationshipUsingNamedParameters;
 use Illuminate\Tests\Integration\Http\Fixtures\PostResourceWithoutWrap;
 use Illuminate\Tests\Integration\Http\Fixtures\PostResourceWithUnlessOptionalData;
 use Illuminate\Tests\Integration\Http\Fixtures\ReallyEmptyPostResource;
@@ -44,7 +46,6 @@ use Illuminate\Tests\Integration\Http\Fixtures\ResourceWithPreservedKeys;
 use Illuminate\Tests\Integration\Http\Fixtures\SerializablePostResource;
 use Illuminate\Tests\Integration\Http\Fixtures\Subscription;
 use LogicException;
-use Mockery as m;
 use Orchestra\Testbench\TestCase;
 
 class ResourceTest extends TestCase
@@ -422,6 +423,59 @@ class ResourceTest extends TestCase
         ]);
     }
 
+    public function testResourcesMayHaveOptionalRelationshipExists()
+    {
+        Route::get('/', function () {
+            return new PostResourceWithOptionalRelationshipExists(new Post([
+                'id' => 5,
+                'title' => 'Test Title',
+            ]));
+        });
+
+        $response = $this->withoutExceptionHandling()->get(
+            '/', ['Accept' => 'application/json']
+        );
+
+        $response->assertStatus(200);
+
+        $response->assertExactJson([
+            'data' => [
+                'id' => 5,
+                'has_favourited_posts' => 'No',
+            ],
+        ]);
+    }
+
+    public function testResourcesMayLoadOptionalRelationshipExists()
+    {
+        Route::get('/', function () {
+            $post = new Post([
+                'id' => 5,
+                'title' => 'Test Title',
+                'authors_exists' => true,
+                'favourited_posts_exists' => true,
+                'comments_exists' => false,
+            ]);
+
+            return new PostResourceWithOptionalRelationshipExists($post);
+        });
+
+        $response = $this->withoutExceptionHandling()->get(
+            '/', ['Accept' => 'application/json']
+        );
+
+        $response->assertStatus(200);
+
+        $response->assertExactJson([
+            'data' => [
+                'id' => 5,
+                'has_authors' => true,
+                'has_favourited_posts' => 'Yes',
+                'comment_exists' => false,
+            ],
+        ]);
+    }
+
     public function testResourcesMayLoadOptionalRelationships()
     {
         Route::get('/', function () {
@@ -610,6 +664,54 @@ class ResourceTest extends TestCase
         ]);
     }
 
+    public function testWhenLoadedUsingNamedDefaultParameterOnMissingRelation()
+    {
+        Route::get('/', function () {
+            $post = new Post(['id' => 1]);
+
+            return new PostResourceWithOptionalRelationshipUsingNamedParameters($post);
+        });
+
+        $response = $this->withoutExceptionHandling()->get(
+            '/', ['Accept' => 'application/json']
+        );
+
+        $response->assertStatus(200);
+
+        $response->assertExactJson([
+            'data' => [
+                'id' => 1,
+                'author_defaulting_to_null' => null,
+                'author_name' => 'Anonymous',
+            ],
+        ]);
+    }
+
+    public function testWhenLoadedUsingNamedDefaultParameterOnLoadedRelation()
+    {
+        Route::get('/', function () {
+            $post = new Post(['id' => 1]);
+            $post->setRelation('author', new Author(['name' => 'jrrmartin']));
+
+            return new PostResourceWithOptionalRelationshipUsingNamedParameters($post);
+        });
+
+        $response = $this->withoutExceptionHandling()->get(
+            '/', ['Accept' => 'application/json']
+        );
+
+        $response->assertStatus(200);
+
+        $response->assertExactJson([
+            'data' => [
+                'id' => 1,
+                'author' => ['name' => 'jrrmartin'],
+                'author_defaulting_to_null' => ['name' => 'jrrmartin'],
+                'author_name' => 'jrrmartin',
+            ],
+        ]);
+    }
+
     public function testResourcesMayHaveOptionalPivotRelationshipsWithCustomAccessor()
     {
         Route::get('/', function () {
@@ -1400,12 +1502,16 @@ class ResourceTest extends TestCase
 
     public function testPostTooLargeException()
     {
-        $this->expectException(PostTooLargeException::class);
-
-        $request = m::mock(Request::class, ['server' => ['CONTENT_LENGTH' => '2147483640']]);
+        $request = new Request(server: ['CONTENT_LENGTH' => '4']);
         $post = new ValidatePostSize;
-        $post->handle($request, function () {
-        });
+        $post->handle($request, fn () => null);
+
+        $this->expectException(PostTooLargeException::class);
+        $this->expectExceptionMessage('The POST data is too large.');
+
+        $request = new Request(server: ['CONTENT_LENGTH' => '2147483640']);
+        $post = new ValidatePostSize;
+        $post->handle($request, fn () => null);
     }
 
     public function testLeadingMergeKeyedValueIsMergedCorrectlyWhenFirstValueIsMissing()
@@ -1723,6 +1829,30 @@ class ResourceTest extends TestCase
         ], ['data' => [0 => 10, 1 => 20, 'total' => 30]]);
     }
 
+    public function testItThrowsNoErrorInStrictModeWhenResourceIsPaginated()
+    {
+        $originalMode = Model::preventsAccessingMissingAttributes();
+        Model::preventAccessingMissingAttributes();
+        try {
+            Route::get('/', function () {
+                $paginator = new LengthAwarePaginator(
+                    collect([new Post(['id' => 5, 'title' => 'Test Title', 'reading_time' => 3.0])]),
+                    10, 15, 1
+                );
+
+                return PostResourceWithJsonOptions::collection($paginator);
+            });
+
+            $response = $this->withoutExceptionHandling()->get(
+                '/', ['Accept' => 'application/json']
+            );
+
+            $response->assertStatus(200);
+        } finally {
+            Model::preventAccessingMissingAttributes($originalMode);
+        }
+    }
+
     private function assertJsonResourceResponse($data, $expectedJson)
     {
         Route::get('/', function () use ($data) {
