@@ -10,8 +10,6 @@ use Orchestra\Testbench\TestCase;
 
 class SendingNotificationsViaAnonymousNotifiableTest extends TestCase
 {
-    public $mailer;
-
     public function testMailIsSent()
     {
         $notifiable = (new AnonymousNotifiable)
@@ -28,6 +26,20 @@ class SendingNotificationsViaAnonymousNotifiableTest extends TestCase
         ], $_SERVER['__notifiable.route']);
     }
 
+    public function testAnonymousNotifiableWithMultipleRoutes()
+    {
+        $_SERVER['__notifiable.route'] = [];
+
+        NotificationFacade::routes([
+            'testchannel' => 'enzo',
+            'anothertestchannel' => 'enzo@deepblue.com',
+        ])->notify(new TestMailNotificationForAnonymousNotifiable());
+
+        $this->assertEquals([
+            'enzo', 'enzo@deepblue.com',
+        ], $_SERVER['__notifiable.route']);
+    }
+
     public function testFaking()
     {
         $fake = NotificationFacade::fake();
