@@ -3,19 +3,19 @@
 namespace Illuminate\Tests\Integration\Queue;
 
 use Illuminate\Support\Env;
+use Orchestra\Testbench\Attributes\RequiresEnv;
+use PHPUnit\Framework\Attributes\RequiresOperatingSystem;
 
+#[RequiresOperatingSystem('Linux|Darwin')]
+#[RequiresEnv('DYNAMODB_ENDPOINT')]
 class DynamoBatchTestWithTTL extends DynamoBatchTest
 {
     protected function defineEnvironment($app)
     {
-        if (is_null($endpoint = Env::get('DYNAMODB_ENDPOINT'))) {
-            $this->markTestSkipped('Require `dynamodb` to be configured');
-        }
-
         $app['config']->set('queue.batching', [
             'driver' => 'dynamodb',
             'region' => 'us-west-2',
-            'endpoint' => $endpoint,
+            'endpoint' => Env::get('DYNAMODB_ENDPOINT'),
             'key' => 'key',
             'secret' => 'secret',
             'ttl' => 1,
