@@ -6,15 +6,19 @@ use Illuminate\Database\Eloquent\Collection;
 use Illuminate\Database\Eloquent\Model;
 use Illuminate\Database\Eloquent\Relations\Pivot;
 use Illuminate\Database\Schema\Blueprint;
+use Illuminate\Foundation\Testing\RefreshDatabase;
 use Illuminate\Queue\Attributes\WithoutRelations;
 use Illuminate\Queue\SerializesModels;
 use LogicException;
+use Orchestra\Testbench\Attributes\WithConfig;
 use Orchestra\Testbench\TestCase;
 use Schema;
 
 class ModelSerializationTest extends TestCase
 {
-    protected function getEnvironmentSetUp($app)
+    use RefreshDatabase;
+
+    protected function defineEnvironment($app)
     {
         $app['config']->set('database.connections.custom', [
             'driver' => 'sqlite',
@@ -63,6 +67,8 @@ class ModelSerializationTest extends TestCase
 
     public function testItSerializeUserOnDefaultConnection()
     {
+        $defaultConnection = config('database.default');
+
         $user = ModelSerializationTestUser::create([
             'email' => 'mohamed@laravel.com',
         ]);
@@ -75,16 +81,16 @@ class ModelSerializationTest extends TestCase
 
         $unSerialized = unserialize($serialized);
 
-        $this->assertSame('testing', $unSerialized->user->getConnectionName());
+        $this->assertSame($defaultConnection, $unSerialized->user->getConnectionName());
         $this->assertSame('mohamed@laravel.com', $unSerialized->user->email);
 
-        $serialized = serialize(new CollectionSerializationTestClass(ModelSerializationTestUser::on('testing')->get()));
+        $serialized = serialize(new CollectionSerializationTestClass(ModelSerializationTestUser::on($defaultConnection)->get()));
 
         $unSerialized = unserialize($serialized);
 
-        $this->assertSame('testing', $unSerialized->users[0]->getConnectionName());
+        $this->assertSame($defaultConnection, $unSerialized->users[0]->getConnectionName());
         $this->assertSame('mohamed@laravel.com', $unSerialized->users[0]->email);
-        $this->assertSame('testing', $unSerialized->users[1]->getConnectionName());
+        $this->assertSame($defaultConnection, $unSerialized->users[1]->getConnectionName());
         $this->assertSame('taylor@laravel.com', $unSerialized->users[1]->email);
     }
 
@@ -279,6 +285,8 @@ class ModelSerializationTest extends TestCase
     {
         require_once __DIR__.'/typed-properties.php';
 
+        $defaultConnection = config('database.default');
+
         $user = ModelSerializationTestUser::create([
             'email' => 'mohamed@laravel.com',
         ]);
@@ -291,21 +299,22 @@ class ModelSerializationTest extends TestCase
 
         $unSerialized = unserialize($serialized);
 
-        $this->assertSame('testing', $unSerialized->user->getConnectionName());
+        $this->assertSame($defaultConnection, $unSerialized->user->getConnectionName());
         $this->assertSame('mohamed@laravel.com', $unSerialized->user->email);
         $this->assertSame(5, $unSerialized->getId());
         $this->assertSame(['James', 'Taylor', 'Mohamed'], $unSerialized->getNames());
 
-        $serialized = serialize(new TypedPropertyCollectionTestClass(ModelSerializationTestUser::on('testing')->get()));
+        $serialized = serialize(new TypedPropertyCollectionTestClass(ModelSerializationTestUser::on($defaultConnection)->get()));
 
         $unSerialized = unserialize($serialized);
 
-        $this->assertSame('testing', $unSerialized->users[0]->getConnectionName());
+        $this->assertSame($defaultConnection, $unSerialized->users[0]->getConnectionName());
         $this->assertSame('mohamed@laravel.com', $unSerialized->users[0]->email);
-        $this->assertSame('testing', $unSerialized->users[1]->getConnectionName());
+        $this->assertSame($defaultConnection, $unSerialized->users[1]->getConnectionName());
         $this->assertSame('taylor@laravel.com', $unSerialized->users[1]->email);
     }
 
+    #[WithConfig('database.default', 'testing')]
     public function test_model_serialization_structure()
     {
         $user = ModelSerializationTestUser::create([
@@ -319,6 +328,7 @@ class ModelSerializationTest extends TestCase
         );
     }
 
+    #[WithConfig('database.default', 'testing')]
     public function test_it_respects_without_relations_attribute()
     {
         $user = User::create([
@@ -332,6 +342,7 @@ class ModelSerializationTest extends TestCase
         );
     }
 
+    #[WithConfig('database.default', 'testing')]
     public function test_it_respects_without_relations_attribute_applied_to_class()
     {
         $user = User::create([
