@@ -8,17 +8,14 @@ use Illuminate\Database\DatabaseTransactionsManager;
 use Illuminate\Foundation\Bus\Dispatchable;
 use Illuminate\Support\Facades\Bus;
 use Mockery as m;
+use Orchestra\Testbench\Attributes\WithConfig;
 use Orchestra\Testbench\TestCase;
 use Throwable;
 
+#[WithConfig('queue.default', 'sqs')]
+#[WithConfig('queue.connections.sqs.after_commit', true)]
 class QueueConnectionTest extends TestCase
 {
-    protected function getEnvironmentSetUp($app)
-    {
-        $app['config']->set('queue.default', 'sqs');
-        $app['config']->set('queue.connections.sqs.after_commit', true);
-    }
-
     protected function tearDown(): void
     {
         QueueConnectionTestJob::$ran = false;
