@@ -6,9 +6,6 @@ use Illuminate\Http\Request;
 use Illuminate\Validation\ValidationException;
 use Orchestra\Testbench\TestCase;
 
-/**
- * @group integration
- */
 class RequestValidationTest extends TestCase
 {
     public function testValidateMacro()
@@ -45,7 +42,7 @@ class RequestValidationTest extends TestCase
         try {
             $request->validateWithBag('some_bag', ['name' => 'string']);
         } catch (ValidationException $validationException) {
-            $this->assertEquals('some_bag', $validationException->errorBag);
+            $this->assertSame('some_bag', $validationException->errorBag);
         }
     }
 }
