@@ -24,13 +24,7 @@ class MailMailableTest extends TestCase
 
     public function testMailableSetsRecipientsCorrectly()
     {
-        Container::getInstance()->instance('mailer', new class
-        {
-            public function render()
-            {
-                //
-            }
-        });
+        $this->stubMailer();
 
         $mailable = new WelcomeMailableStub;
         $mailable->to('taylor@laravel.com');
@@ -114,13 +108,7 @@ class MailMailableTest extends TestCase
 
     public function testMailableSetsCcRecipientsCorrectly()
     {
-        Container::getInstance()->instance('mailer', new class
-        {
-            public function render()
-            {
-                //
-            }
-        });
+        $this->stubMailer();
 
         $mailable = new WelcomeMailableStub;
         $mailable->cc('taylor@laravel.com');
@@ -211,13 +199,7 @@ class MailMailableTest extends TestCase
 
     public function testMailableSetsBccRecipientsCorrectly()
     {
-        Container::getInstance()->instance('mailer', new class
-        {
-            public function render()
-            {
-                //
-            }
-        });
+        $this->stubMailer();
 
         $mailable = new WelcomeMailableStub;
         $mailable->bcc('taylor@laravel.com');
@@ -308,13 +290,7 @@ class MailMailableTest extends TestCase
 
     public function testMailableSetsReplyToCorrectly()
     {
-        Container::getInstance()->instance('mailer', new class
-        {
-            public function render()
-            {
-                //
-            }
-        });
+        $this->stubMailer();
 
         $mailable = new WelcomeMailableStub;
         $mailable->replyTo('taylor@laravel.com');
@@ -394,13 +370,7 @@ class MailMailableTest extends TestCase
 
     public function testMailableSetsFromCorrectly()
     {
-        Container::getInstance()->instance('mailer', new class
-        {
-            public function render()
-            {
-                //
-            }
-        });
+        $this->stubMailer();
 
         $mailable = new WelcomeMailableStub;
         $mailable->from('taylor@laravel.com');
@@ -592,6 +562,7 @@ class MailMailableTest extends TestCase
             'first_name' => 'Taylor',
             'lastName' => 'Otwell',
             'framework' => 'Laravel',
+            '__laravel_mailable' => get_class($mailable),
         ];
 
         $this->assertSame($expected, $mailable->buildViewData());
@@ -629,13 +600,7 @@ class MailMailableTest extends TestCase
 
     public function testMailableMetadataGetsSent()
     {
-        Container::getInstance()->instance('mailer', new class
-        {
-            public function render()
-            {
-                //
-            }
-        });
+        $this->stubMailer();
 
         $view = m::mock(Factory::class);
 
@@ -670,13 +635,7 @@ class MailMailableTest extends TestCase
 
     public function testMailableTagGetsSent()
     {
-        Container::getInstance()->instance('mailer', new class
-        {
-            public function render()
-            {
-                //
-            }
-        });
+        $this->stubMailer();
 
         $view = m::mock(Factory::class);
 
@@ -828,13 +787,7 @@ class MailMailableTest extends TestCase
 
     public function testHasAttachmentWithEnvelopeAttachments()
     {
-        Container::getInstance()->instance('mailer', new class
-        {
-            public function render()
-            {
-                //
-            }
-        });
+        $this->stubMailer();
         $mailable = new class extends Mailable
         {
             public function envelope()
@@ -1033,13 +986,7 @@ class MailMailableTest extends TestCase
 
     public function testAssertHasAttachment()
     {
-        Container::getInstance()->instance('mailer', new class
-        {
-            public function render()
-            {
-                //
-            }
-        });
+        $this->stubMailer();
 
         $mailable = new class() extends Mailable
         {
@@ -1069,13 +1016,7 @@ class MailMailableTest extends TestCase
 
     public function testAssertHasAttachedData()
     {
-        Container::getInstance()->instance('mailer', new class
-        {
-            public function render()
-            {
-                //
-            }
-        });
+        $this->stubMailer();
 
         $mailable = new class() extends Mailable
         {
@@ -1133,13 +1074,7 @@ class MailMailableTest extends TestCase
 
     public function testAssertHasSubject()
     {
-        Container::getInstance()->instance('mailer', new class
-        {
-            public function render()
-            {
-                //
-            }
-        });
+        $this->stubMailer();
 
         $mailable = new class() extends Mailable
         {
@@ -1193,13 +1128,7 @@ class MailMailableTest extends TestCase
 
     public function testMailableAttributesInBuild()
     {
-        Container::getInstance()->instance('mailer', new class
-        {
-            public function render()
-            {
-                //
-            }
-        });
+        $this->stubMailer();
 
         $mailable = new class() extends Mailable
         {
@@ -1226,6 +1155,32 @@ class MailMailableTest extends TestCase
         $mailable->assertHasMetadata('user_id', 1);
         $mailable->assertHasSubject('test subject');
     }
+
+    public function testMailablesCanBeTapped()
+    {
+        $this->stubMailer();
+
+        $mail = new WelcomeMailableStub;
+
+        $mail->tap(fn ($mailable) => $mailable->to('taylor@laravel.com', 'Taylor Otwell'));
+        $mail->tap(fn ($mailable) => $mailable->subject('Test Subject!'));
+
+        $mail->tap(function ($mailable) {
+            $mailable->assertTo('taylor@laravel.com')
+                ->assertHasSubject('Test Subject!');
+        });
+    }
+
+    protected function stubMailer()
+    {
+        Container::getInstance()->instance('mailer', new class
+        {
+            public function render()
+            {
+                //
+            }
+        });
+    }
 }
 
 class MailableHeadersStub extends Mailable
@@ -1254,7 +1209,7 @@ class WelcomeMailableStub extends Mailable
     public function build()
     {
         $this->with('first_name', 'Taylor')
-             ->withLastName('Otwell');
+            ->withLastName('Otwell');
     }
 }
 
