@@ -2,6 +2,7 @@
 
 namespace Illuminate\Tests\Notifications;
 
+use Carbon\Carbon;
 use Illuminate\Notifications\Channels\DatabaseChannel;
 use Illuminate\Notifications\Messages\DatabaseMessage;
 use Illuminate\Notifications\Notification;
@@ -57,10 +58,10 @@ class NotificationDatabaseChannelTest extends TestCase
         $notifiable = m::mock();
 
         $notifiable->shouldReceive('routeNotificationFor->create')->with([
-            'id'        => 1,
-            'type'      => 'MONTHLY',
-            'data'      => ['invoice_id' => 1],
-            'read_at'   => null,
+            'id' => 1,
+            'type' => 'MONTHLY',
+            'data' => ['invoice_id' => 1],
+            'read_at' => Carbon::now()->toDateTimeString(),
             'something' => 'else',
         ]);
 
@@ -88,6 +89,11 @@ class NotificationDatabaseChannelCustomizeTypeTestNotification extends Notificat
     {
         return 'MONTHLY';
     }
+
+    public function initialDatabaseReadAtValue()
+    {
+        return Carbon::now();
+    }
 }
 
 class ExtendedDatabaseChannel extends DatabaseChannel
