@@ -6,8 +6,12 @@ use Illuminate\Container\Container;
 use Illuminate\Database\Capsule\Manager as DB;
 use Illuminate\Database\Eloquent\Model as Eloquent;
 use Illuminate\Database\Schema\Blueprint;
+use Illuminate\Events\Dispatcher;
 use Illuminate\Queue\DatabaseQueue;
+use Illuminate\Queue\Events\JobQueued;
+use Illuminate\Queue\Events\JobQueueing;
 use Illuminate\Support\Carbon;
+use Illuminate\Support\Str;
 use PHPUnit\Framework\TestCase;
 
 class QueueDatabaseQueueIntegrationTest extends TestCase
@@ -44,7 +48,9 @@ class QueueDatabaseQueueIntegrationTest extends TestCase
 
         $this->queue = new DatabaseQueue($this->connection(), $this->table);
 
-        $this->container = $this->createMock(Container::class);
+        $this->container = new Container;
+
+        $this->container->instance('events', new Dispatcher($this->container));
 
         $this->queue->setContainer($this->container);
 
@@ -241,4 +247,29 @@ class QueueDatabaseQueueIntegrationTest extends TestCase
 
         $this->assertNull($popped_job);
     }
+
+    public function testJobPayloadIsAvailableOnEvents()
+    {
+        $jobQueueingEvent = null;
+        $jobQueuedEvent = null;
+        Str::createUuidsUsingSequence([
+            'expected-job-uuid',
+        ]);
+        $this->container['events']->listen(function (JobQueueing $e) use (&$jobQueueingEvent) {
+            $jobQueueingEvent = $e;
+        });
+        $this->container['events']->listen(function (JobQueued $e) use (&$jobQueuedEvent) {
+            $jobQueuedEvent = $e;
+        });
+
+        $this->queue->push('MyJob', [
+            'laravel' => 'Framework',
+        ]);
+
+        $this->assertIsArray($jobQueueingEvent->payload());
+        $this->assertSame('expected-job-uuid', $jobQueueingEvent->payload()['uuid']);
+
+        $this->assertIsArray($jobQueuedEvent->payload());
+        $this->assertSame('expected-job-uuid', $jobQueuedEvent->payload()['uuid']);
+    }
 }
