@@ -8,6 +8,8 @@ use Illuminate\Contracts\Debug\ExceptionHandler;
 use Illuminate\Contracts\Events\Dispatcher;
 use Illuminate\Contracts\Queue\Job as QueueJobContract;
 use Illuminate\Queue\Events\JobExceptionOccurred;
+use Illuminate\Queue\Events\JobPopped;
+use Illuminate\Queue\Events\JobPopping;
 use Illuminate\Queue\Events\JobProcessed;
 use Illuminate\Queue\Events\JobProcessing;
 use Illuminate\Queue\MaxAttemptsExceededException;
@@ -23,6 +25,7 @@ class QueueWorkerTest extends TestCase
 {
     public $events;
     public $exceptionHandler;
+    public $maintenanceFlags;
 
     protected function setUp(): void
     {
@@ -37,7 +40,11 @@ class QueueWorkerTest extends TestCase
 
     protected function tearDown(): void
     {
-        Container::setInstance(null);
+        parent::tearDown();
+
+        Carbon::setTestNow();
+
+        Container::setInstance();
     }
 
     public function testJobCanBeFired()
@@ -45,6 +52,8 @@ class QueueWorkerTest extends TestCase
         $worker = $this->getWorker('default', ['queue' => [$job = new WorkerFakeJob]]);
         $worker->runNextJob('default', 'queue', new WorkerOptions);
         $this->assertTrue($job->fired);
+        $this->events->shouldHaveReceived('dispatch')->with(m::type(JobPopping::class))->once();
+        $this->events->shouldHaveReceived('dispatch')->with(m::type(JobPopped::class))->once();
         $this->events->shouldHaveReceived('dispatch')->with(m::type(JobProcessing::class))->once();
         $this->events->shouldHaveReceived('dispatch')->with(m::type(JobProcessed::class))->once();
     }
@@ -113,7 +122,7 @@ class QueueWorkerTest extends TestCase
     public function testExceptionIsReportedIfConnectionThrowsExceptionOnJobPop()
     {
         $worker = new InsomniacWorker(
-            new WorkerFakeManager('default', new BrokenQueueConnection($e = new RuntimeException)),
+            new WorkerFakeManager('default', new BrokenQueueConnection('default', $e = new RuntimeException)),
             $this->events,
             $this->exceptionHandler,
             function () {
@@ -305,7 +314,7 @@ class QueueWorkerTest extends TestCase
             $worker->daemon('default', 'queue', $this->workerOptions());
 
             $this->fail('Expected LoopBreakerException to be thrown');
-        } catch (LoopBreakerException $e) {
+        } catch (LoopBreakerException) {
             $this->assertSame(1, $firstJob->attempts);
 
             $this->assertSame(0, $secondJob->attempts);
@@ -372,7 +381,7 @@ class QueueWorkerTest extends TestCase
     private function workerDependencies($connectionName = 'default', $jobs = [], ?callable $isInMaintenanceMode = null)
     {
         return [
-            new WorkerFakeManager($connectionName, new WorkerFakeConnection($jobs)),
+            new WorkerFakeManager($connectionName, new WorkerFakeConnection($connectionName, $jobs)),
             $this->events,
             $this->exceptionHandler,
             $isInMaintenanceMode ?? function () {
@@ -406,7 +415,7 @@ class InsomniacWorker extends Worker
         $this->sleptFor = $seconds;
     }
 
-    public function stop($status = 0)
+    public function stop($status = 0, $options = null)
     {
         return $status;
     }
@@ -439,10 +448,12 @@ class WorkerFakeManager extends QueueManager
 
 class WorkerFakeConnection
 {
+    public $connectionName;
     public $jobs = [];
 
-    public function __construct($jobs)
+    public function __construct($connectionName, $jobs)
     {
+        $this->connectionName = $connectionName;
         $this->jobs = $jobs;
     }
 
@@ -450,14 +461,21 @@ class WorkerFakeConnection
     {
         return array_shift($this->jobs[$queue]);
     }
+
+    public function getConnectionName()
+    {
+        return $this->connectionName;
+    }
 }
 
 class BrokenQueueConnection
 {
+    public $connectionName;
     public $exception;
 
-    public function __construct($exception)
+    public function __construct($connectionName, $exception)
     {
+        $this->connectionName = $connectionName;
         $this->exception = $exception;
     }
 
@@ -465,6 +483,11 @@ class BrokenQueueConnection
     {
         throw $this->exception;
     }
+
+    public function getConnectionName()
+    {
+        return $this->connectionName;
+    }
 }
 
 class WorkerFakeJob implements QueueJobContract
