@@ -6,6 +6,7 @@ use Illuminate\Foundation\Application;
 use Illuminate\Foundation\Testing\Concerns\InteractsWithRedis;
 use Illuminate\Redis\RedisManager;
 use PHPUnit\Framework\TestCase;
+use Redis;
 
 class RedisConnectorTest extends TestCase
 {
@@ -207,4 +208,88 @@ class RedisConnectorTest extends TestCase
         $parameters = $predisClient->getConnection()->getSentinelConnection()->getParameters();
         $this->assertEquals($host, $parameters->host);
     }
+
+    public function testPrefixOverrideBehaviour()
+    {
+        $host = env('REDIS_HOST', '127.0.0.1');
+        $port = env('REDIS_PORT', 6379);
+
+        $predis1 = new RedisManager(new Application, 'predis', [
+            'cluster' => false,
+            'options' => [
+                'prefix' => 'test_',
+            ],
+            'default' => [
+                'scheme' => 'tls',
+                'host' => $host,
+                'port' => $port,
+                'database' => 5,
+                'timeout' => 0.5,
+                'options' => [
+                    'prefix' => 'test_default_options_',
+                ],
+            ],
+        ]);
+        $predisClient1 = $predis1->client();
+        $this->assertEquals('test_default_options_', $predisClient1->getOptions()->prefix->getPrefix());
+
+        $predis2 = new RedisManager(new Application, 'predis', [
+            'cluster' => false,
+            'options' => [
+                'prefix' => 'test_',
+            ],
+            'default' => [
+                'scheme' => 'tls',
+                'host' => $host,
+                'port' => $port,
+                'database' => 5,
+                'timeout' => 0.5,
+                'options' => [
+                    'prefix' => 'test_default_options_',
+                ],
+                'prefix' => 'test_default_config_',
+            ],
+        ]);
+        $predisClient2 = $predis2->client();
+        $this->assertEquals('test_default_config_', $predisClient2->getOptions()->prefix->getPrefix());
+
+        $phpRedis1 = new RedisManager(new Application, 'phpredis', [
+            'cluster' => false,
+            'options' => [
+                'prefix' => 'test_',
+            ],
+            'default' => [
+                'scheme' => 'tcp',
+                'host' => $host,
+                'port' => $port,
+                'database' => 5,
+                'timeout' => 0.5,
+                'options' => [
+                    'prefix' => 'test_default_options_',
+                ],
+            ],
+        ]);
+        $phpRedisClient1 = $phpRedis1->connection()->client();
+        $this->assertEquals('test_default_options_', $phpRedisClient1->getOption(Redis::OPT_PREFIX));
+
+        $phpRedis2 = new RedisManager(new Application, 'phpredis', [
+            'cluster' => false,
+            'options' => [
+                'prefix' => 'test_',
+            ],
+            'default' => [
+                'scheme' => 'tcp',
+                'host' => $host,
+                'port' => $port,
+                'database' => 5,
+                'timeout' => 0.5,
+                'options' => [
+                    'prefix' => 'test_default_options_',
+                ],
+                'prefix' => 'test_default_config_',
+            ],
+        ]);
+        $phpRedisClient2 = $phpRedis2->connection()->client();
+        $this->assertEquals('test_default_config_', $phpRedisClient2->getOption(Redis::OPT_PREFIX));
+    }
 }
