@@ -5,7 +5,6 @@ namespace Illuminate\Tests\Routing;
 use Illuminate\Database\Eloquent\Model;
 use Illuminate\Routing\RouteAction;
 use Laravel\SerializableClosure\SerializableClosure;
-use Opis\Closure\SerializableClosure as OpisSerializableClosure;
 use PHPUnit\Framework\TestCase;
 
 class RouteActionTest extends TestCase
@@ -16,9 +15,8 @@ class RouteActionTest extends TestCase
             return $user;
         };
 
-        $action = ['uses' => serialize(\PHP_VERSION_ID < 70400
-            ? new OpisSerializableClosure($callable)
-            : new SerializableClosure($callable)
+        $action = ['uses' => serialize(
+            new SerializableClosure($callable)
         )];
 
         $this->assertTrue(RouteAction::containsSerializedClosure($action));
