@@ -12,3 +12,9 @@ enum TestBackedEnum: int
     case A = 1;
     case B = 2;
 }
+
+enum TestStringBackedEnum: string
+{
+    case A = 'A';
+    case B = 'B';
+}
