@@ -32,6 +32,28 @@ class SleepTest extends TestCase
         $this->assertEqualsWithDelta(1, $end - $start, 0.03);
     }
 
+    public function testCallbacksMayBeExecutedUsingThen()
+    {
+        $this->assertEquals(123, Sleep::for(1)->milliseconds()->then(fn () => 123));
+    }
+
+    public function testSleepRespectsWhile()
+    {
+        $_SERVER['__sleep.while'] = 0;
+
+        $result = Sleep::for(10)->milliseconds()->while(function () {
+            static $results = [true, true, false];
+            $_SERVER['__sleep.while']++;
+
+            return array_shift($results);
+        })->then(fn () => 100);
+
+        $this->assertEquals(3, $_SERVER['__sleep.while']);
+        $this->assertEquals(100, $result);
+
+        unset($_SERVER['__sleep.while']);
+    }
+
     public function testItSleepsForSecondsWithMilliseconds()
     {
         $start = microtime(true);
@@ -58,7 +80,7 @@ class SleepTest extends TestCase
 
         $sleep = Sleep::for(1.5)->minutes();
 
-        $this->assertSame($sleep->duration->totalMicroseconds, 90_000_000);
+        $this->assertSame((float) $sleep->duration->totalMicroseconds, 90_000_000.0);
     }
 
     public function testItCanSpecifyMinute()
@@ -67,7 +89,7 @@ class SleepTest extends TestCase
 
         $sleep = Sleep::for(1)->minute();
 
-        $this->assertSame($sleep->duration->totalMicroseconds, 60_000_000);
+        $this->assertSame((float) $sleep->duration->totalMicroseconds, 60_000_000.0);
     }
 
     public function testItCanSpecifySeconds()
@@ -76,7 +98,7 @@ class SleepTest extends TestCase
 
         $sleep = Sleep::for(1.5)->seconds();
 
-        $this->assertSame($sleep->duration->totalMicroseconds, 1_500_000);
+        $this->assertSame((float) $sleep->duration->totalMicroseconds, 1_500_000.0);
     }
 
     public function testItCanSpecifySecond()
@@ -85,7 +107,7 @@ class SleepTest extends TestCase
 
         $sleep = Sleep::for(1)->second();
 
-        $this->assertSame($sleep->duration->totalMicroseconds, 1_000_000);
+        $this->assertSame((float) $sleep->duration->totalMicroseconds, 1_000_000.0);
     }
 
     public function testItCanSpecifyMilliseconds()
@@ -94,7 +116,7 @@ class SleepTest extends TestCase
 
         $sleep = Sleep::for(1.5)->milliseconds();
 
-        $this->assertSame($sleep->duration->totalMicroseconds, 1_500);
+        $this->assertSame((float) $sleep->duration->totalMicroseconds, 1_500.0);
     }
 
     public function testItCanSpecifyMillisecond()
@@ -103,7 +125,7 @@ class SleepTest extends TestCase
 
         $sleep = Sleep::for(1)->millisecond();
 
-        $this->assertSame($sleep->duration->totalMicroseconds, 1_000);
+        $this->assertSame((float) $sleep->duration->totalMicroseconds, 1_000.0);
     }
 
     public function testItCanSpecifyMicroseconds()
@@ -113,7 +135,7 @@ class SleepTest extends TestCase
         $sleep = Sleep::for(1.5)->microseconds();
 
         // rounded as microseconds is the smallest unit supported...
-        $this->assertSame($sleep->duration->totalMicroseconds, 1);
+        $this->assertSame((float) $sleep->duration->totalMicroseconds, 1.0);
     }
 
     public function testItCanSpecifyMicrosecond()
@@ -122,7 +144,7 @@ class SleepTest extends TestCase
 
         $sleep = Sleep::for(1)->microsecond();
 
-        $this->assertSame($sleep->duration->totalMicroseconds, 1);
+        $this->assertSame((float) $sleep->duration->totalMicroseconds, 1.0);
     }
 
     public function testItCanChainDurations()
@@ -130,9 +152,9 @@ class SleepTest extends TestCase
         Sleep::fake();
 
         $sleep = Sleep::for(1)->second()
-                      ->and(500)->microseconds();
+            ->and(500)->microseconds();
 
-        $this->assertSame($sleep->duration->totalMicroseconds, 1000500);
+        $this->assertSame((float) $sleep->duration->totalMicroseconds, 1000500.0);
     }
 
     public function testItCanUseDateInterval()
@@ -141,7 +163,7 @@ class SleepTest extends TestCase
 
         $sleep = Sleep::for(CarbonInterval::seconds(1)->addMilliseconds(5));
 
-        $this->assertSame($sleep->duration->totalMicroseconds, 1_005_000);
+        $this->assertSame((float) $sleep->duration->totalMicroseconds, 1_005_000.0);
     }
 
     public function testItThrowsForUnknownTimeUnit()
@@ -427,17 +449,17 @@ class SleepTest extends TestCase
 
         Sleep::for(5)->seconds();
 
-        Sleep::assertSlept(fn (CarbonInterval $duration) => $duration->totalSeconds === 5);
+        Sleep::assertSlept(fn (CarbonInterval $duration) => (float) $duration->totalSeconds === 5.0);
 
         try {
-            Sleep::assertSlept(fn (CarbonInterval $duration) => $duration->totalSeconds === 5, 2);
+            Sleep::assertSlept(fn (CarbonInterval $duration) => (float) $duration->totalSeconds === 5.0, 2);
             $this->fail();
         } catch (AssertionFailedError $e) {
             $this->assertSame("The expected sleep was found [1] times instead of [2].\nFailed asserting that 1 is identical to 2.", $e->getMessage());
         }
 
         try {
-            Sleep::assertSlept(fn (CarbonInterval $duration) => $duration->totalSeconds === 6);
+            Sleep::assertSlept(fn (CarbonInterval $duration) => (float) $duration->totalSeconds === 6.0);
             $this->fail();
         } catch (AssertionFailedError $e) {
             $this->assertSame("The expected sleep was found [0] times instead of [1].\nFailed asserting that 0 is identical to 1.", $e->getMessage());
@@ -464,15 +486,15 @@ class SleepTest extends TestCase
 
         // A static macro can be referenced
         $sleep = Sleep::forSomeConfiguredAmountOfTime();
-        $this->assertSame($sleep->duration->totalMicroseconds, 3000000);
+        $this->assertSame((float) $sleep->duration->totalMicroseconds, 3000000.0);
 
         // A macro can specify a new duration
         $sleep = $sleep->useSomeOtherAmountOfTime();
-        $this->assertSame($sleep->duration->totalMicroseconds, 1234000);
+        $this->assertSame((float) $sleep->duration->totalMicroseconds, 1234000.0);
 
         // A macro can supplement an existing duration
         $sleep = $sleep->andSomeMoreGranularControl();
-        $this->assertSame($sleep->duration->totalMicroseconds, 1234567);
+        $this->assertSame((float) $sleep->duration->totalMicroseconds, 1234567.0);
     }
 
     public function testItCanReplacePreviouslyDefinedDurations()
@@ -484,13 +506,13 @@ class SleepTest extends TestCase
         });
 
         $sleep = Sleep::for(1)->second();
-        $this->assertSame($sleep->duration->totalMicroseconds, 1000000);
+        $this->assertSame((float) $sleep->duration->totalMicroseconds, 1000000.0);
 
         $sleep->setDuration(2)->second();
-        $this->assertSame($sleep->duration->totalMicroseconds, 2000000);
+        $this->assertSame((float) $sleep->duration->totalMicroseconds, 2000000.0);
 
         $sleep->setDuration(500)->milliseconds();
-        $this->assertSame($sleep->duration->totalMicroseconds, 500000);
+        $this->assertSame((float) $sleep->duration->totalMicroseconds, 500000.0);
     }
 
     public function testItCanSleepConditionallyWhen()
@@ -551,8 +573,8 @@ class SleepTest extends TestCase
             Sleep::for(2)->millisecond(),
         ]);
 
-        $this->assertSame(3, $countA);
-        $this->assertSame(3, $countB);
+        $this->assertSame(3.0, (float) $countA);
+        $this->assertSame(3.0, (float) $countB);
     }
 
     public function testItDoesntRunCallbacksWhenNotFaking()
