@@ -12,7 +12,7 @@ use stdClass;
 
 class SupportArrTest extends TestCase
 {
-    public function testAccessible()
+    public function testAccessible(): void
     {
         $this->assertTrue(Arr::accessible([]));
         $this->assertTrue(Arr::accessible([1, 2]));
@@ -23,6 +23,11 @@ class SupportArrTest extends TestCase
         $this->assertFalse(Arr::accessible('abc'));
         $this->assertFalse(Arr::accessible(new stdClass));
         $this->assertFalse(Arr::accessible((object) ['a' => 1, 'b' => 2]));
+        $this->assertFalse(Arr::accessible(123));
+        $this->assertFalse(Arr::accessible(12.34));
+        $this->assertFalse(Arr::accessible(true));
+        $this->assertFalse(Arr::accessible(new \DateTime));
+        $this->assertFalse(Arr::accessible(static fn () => null));
     }
 
     public function testAdd()
@@ -34,15 +39,34 @@ class SupportArrTest extends TestCase
         $this->assertEquals(['developer' => ['name' => 'Ferid']], Arr::add([], 'developer.name', 'Ferid'));
         $this->assertEquals([1 => 'hAz'], Arr::add([], 1, 'hAz'));
         $this->assertEquals([1 => [1 => 'hAz']], Arr::add([], 1.1, 'hAz'));
+
+        // Case where the key already exists
+        $this->assertEquals(['type' => 'Table'], Arr::add(['type' => 'Table'], 'type', 'Chair'));
+        $this->assertEquals(['category' => ['type' => 'Table']], Arr::add(['category' => ['type' => 'Table']], 'category.type', 'Chair'));
     }
 
     public function testCollapse()
     {
+        // Normal case: a two-dimensional array with different elements
         $data = [['foo', 'bar'], ['baz']];
         $this->assertEquals(['foo', 'bar', 'baz'], Arr::collapse($data));
 
-        $array = [[1], [2], [3], ['foo', 'bar'], collect(['baz', 'boom'])];
-        $this->assertEquals([1, 2, 3, 'foo', 'bar', 'baz', 'boom'], Arr::collapse($array));
+        // Case including numeric and string elements
+        $array = [[1], [2], [3], ['foo', 'bar']];
+        $this->assertEquals([1, 2, 3, 'foo', 'bar'], Arr::collapse($array));
+
+        // Case with empty two-dimensional arrays
+        $emptyArray = [[], [], []];
+        $this->assertEquals([], Arr::collapse($emptyArray));
+
+        // Case with both empty arrays and arrays with elements
+        $mixedArray = [[], [1, 2], [], ['foo', 'bar']];
+        $this->assertEquals([1, 2, 'foo', 'bar'], Arr::collapse($mixedArray));
+
+        // Case including collections and arrays
+        $collection = collect(['baz', 'boom']);
+        $mixedArray = [[1], [2], [3], ['foo', 'bar'], $collection];
+        $this->assertEquals([1, 2, 3, 'foo', 'bar', 'baz', 'boom'], Arr::collapse($mixedArray));
     }
 
     public function testCrossJoin()
@@ -90,11 +114,42 @@ class SupportArrTest extends TestCase
         $this->assertSame([[]], Arr::crossJoin());
     }
 
-    public function testDivide()
+    public function testDivide(): void
     {
+        // Test dividing an empty array
+        [$keys, $values] = Arr::divide([]);
+        $this->assertEquals([], $keys);
+        $this->assertEquals([], $values);
+
+        // Test dividing an array with a single key-value pair
         [$keys, $values] = Arr::divide(['name' => 'Desk']);
         $this->assertEquals(['name'], $keys);
         $this->assertEquals(['Desk'], $values);
+
+        // Test dividing an array with multiple key-value pairs
+        [$keys, $values] = Arr::divide(['name' => 'Desk', 'price' => 100, 'available' => true]);
+        $this->assertEquals(['name', 'price', 'available'], $keys);
+        $this->assertEquals(['Desk', 100, true], $values);
+
+        // Test dividing an array with numeric keys
+        [$keys, $values] = Arr::divide([0 => 'first', 1 => 'second']);
+        $this->assertEquals([0, 1], $keys);
+        $this->assertEquals(['first', 'second'], $values);
+
+        // Test dividing an array with null key
+        [$keys, $values] = Arr::divide([null => 'Null', 1 => 'one']);
+        $this->assertEquals([null, 1], $keys);
+        $this->assertEquals(['Null', 'one'], $values);
+
+        // Test dividing an array where the keys are arrays
+        [$keys, $values] = Arr::divide([['one' => 1, 2 => 'second'], 1 => 'one']);
+        $this->assertEquals([0, 1], $keys);
+        $this->assertEquals([['one' => 1, 2 => 'second'], 'one'], $values);
+
+        // Test dividing an array where the values are arrays
+        [$keys, $values] = Arr::divide([null => ['one' => 1, 2 => 'second'], 1 => 'one']);
+        $this->assertEquals([null, 1], $keys);
+        $this->assertEquals([['one' => 1, 2 => 'second'], 'one'], $values);
     }
 
     public function testDot()
@@ -198,10 +253,22 @@ class SupportArrTest extends TestCase
         $this->assertFalse(Arr::exists(new Collection(['a' => null]), 'b'));
     }
 
-    public function testWhereNotNull()
+    public function testWhereNotNull(): void
     {
         $array = array_values(Arr::whereNotNull([null, 0, false, '', null, []]));
         $this->assertEquals([0, false, '', []], $array);
+
+        $array = array_values(Arr::whereNotNull([1, 2, 3]));
+        $this->assertEquals([1, 2, 3], $array);
+
+        $array = array_values(Arr::whereNotNull([null, null, null]));
+        $this->assertEquals([], $array);
+
+        $array = array_values(Arr::whereNotNull(['a', null, 'b', null, 'c']));
+        $this->assertEquals(['a', 'b', 'c'], $array);
+
+        $array = array_values(Arr::whereNotNull([null, 1, 'string', 0.0, false, [], $class = new stdClass(), $function = fn () => null]));
+        $this->assertEquals([1, 'string', 0.0, false, [], $class, $function], $array);
     }
 
     public function testFirst()
@@ -441,6 +508,10 @@ class SupportArrTest extends TestCase
         $this->assertSame('dayle', Arr::get($array, 'names.otherDeveloper', function () {
             return 'dayle';
         }));
+
+        // Test array has a null key
+        $this->assertSame('bar', Arr::get(['' => 'bar'], ''));
+        $this->assertSame('bar', Arr::get(['' => ['' => 'bar']], '.'));
     }
 
     public function testHas()
@@ -556,6 +627,8 @@ class SupportArrTest extends TestCase
         $this->assertTrue(Arr::isList([0 => 'foo', 'bar']));
         $this->assertTrue(Arr::isList([0 => 'foo', 1 => 'bar']));
 
+        $this->assertFalse(Arr::isList([-1 => 1]));
+        $this->assertFalse(Arr::isList([-1 => 1, 0 => 2]));
         $this->assertFalse(Arr::isList([1 => 'foo', 'bar']));
         $this->assertFalse(Arr::isList([1 => 'foo', 0 => 'bar']));
         $this->assertFalse(Arr::isList([0 => 'foo', 'bar' => 'baz']));
@@ -569,6 +642,17 @@ class SupportArrTest extends TestCase
         $array = Arr::only($array, ['name', 'price']);
         $this->assertEquals(['name' => 'Desk', 'price' => 100], $array);
         $this->assertEmpty(Arr::only($array, ['nonExistingKey']));
+
+        $this->assertEmpty(Arr::only($array, null));
+
+        // Test with array having numeric keys
+        $this->assertEquals(['foo'], Arr::only(['foo', 'bar', 'baz'], 0));
+        $this->assertEquals([1 => 'bar', 2 => 'baz'], Arr::only(['foo', 'bar', 'baz'], [1, 2]));
+        $this->assertEmpty(Arr::only(['foo', 'bar', 'baz'], [3]));
+
+        // Test with array having numeric key and string key
+        $this->assertEquals(['foo'], Arr::only(['foo', 'bar' => 'baz'], 0));
+        $this->assertEquals(['bar' => 'baz'], Arr::only(['foo', 'bar' => 'baz'], 'bar'));
     }
 
     public function testPluck()
@@ -706,6 +790,23 @@ class SupportArrTest extends TestCase
         $this->assertEquals(['first' => 'taylor', 'last' => 'otwell'], $data);
     }
 
+    public function testMapWithEmptyArray()
+    {
+        $mapped = Arr::map([], static function ($value, $key) {
+            return $key.'-'.$value;
+        });
+        $this->assertEquals([], $mapped);
+    }
+
+    public function testMapNullValues()
+    {
+        $data = ['first' => 'taylor', 'last' => null];
+        $mapped = Arr::map($data, static function ($value, $key) {
+            return $key.'-'.$value;
+        });
+        $this->assertEquals(['first' => 'first-taylor', 'last' => 'last-'], $mapped);
+    }
+
     public function testMapWithKeys()
     {
         $data = [
@@ -733,6 +834,21 @@ class SupportArrTest extends TestCase
         $this->assertEquals(['first' => 'taylor', 'last' => 'otwell'], $data);
     }
 
+    public function testMapSpread()
+    {
+        $c = [[1, 'a'], [2, 'b']];
+
+        $result = Arr::mapSpread($c, function ($number, $character) {
+            return "{$number}-{$character}";
+        });
+        $this->assertEquals(['1-a', '2-b'], $result);
+
+        $result = Arr::mapSpread($c, function ($number, $character, $key) {
+            return "{$number}-{$character}-{$key}";
+        });
+        $this->assertEquals(['1-a-0', '2-b-1'], $result);
+    }
+
     public function testPrepend()
     {
         $array = Arr::prepend(['one', 'two', 'three', 'four'], 'zero');
@@ -743,6 +859,21 @@ class SupportArrTest extends TestCase
 
         $array = Arr::prepend(['one' => 1, 'two' => 2], 0, null);
         $this->assertEquals([null => 0, 'one' => 1, 'two' => 2], $array);
+
+        $array = Arr::prepend(['one', 'two'], null, '');
+        $this->assertEquals(['' => null, 'one', 'two'], $array);
+
+        $array = Arr::prepend([], 'zero');
+        $this->assertEquals(['zero'], $array);
+
+        $array = Arr::prepend([''], 'zero');
+        $this->assertEquals(['zero', ''], $array);
+
+        $array = Arr::prepend(['one', 'two'], ['zero']);
+        $this->assertEquals([['zero'], 'one', 'two'], $array);
+
+        $array = Arr::prepend(['one', 'two'], ['zero'], 'key');
+        $this->assertEquals(['key' => ['zero'], 'one', 'two'], $array);
     }
 
     public function testPull()
@@ -907,14 +1038,35 @@ class SupportArrTest extends TestCase
         $this->assertEquals([1 => 'hAz'], Arr::set($array, 1, 'hAz'));
     }
 
-    public function testShuffleWithSeed()
+    public function testShuffleProducesDifferentShuffles()
     {
-        $this->assertEquals(
-            Arr::shuffle(range(0, 100, 10), 1234),
-            Arr::shuffle(range(0, 100, 10), 1234)
+        $input = range('a', 'z');
+
+        $this->assertFalse(
+            Arr::shuffle($input) === Arr::shuffle($input) && Arr::shuffle($input) === Arr::shuffle($input),
+            "The shuffles produced the same output each time, which shouldn't happen."
         );
     }
 
+    public function testShuffleActuallyShuffles()
+    {
+        $input = range('a', 'z');
+
+        $this->assertFalse(
+            Arr::shuffle($input) === $input && Arr::shuffle($input) === $input,
+            "The shuffles were unshuffled each time, which shouldn't happen."
+        );
+    }
+
+    public function testShuffleKeepsSameValues()
+    {
+        $input = range('a', 'z');
+        $shuffled = Arr::shuffle($input);
+        sort($shuffled);
+
+        $this->assertEquals($input, $shuffled);
+    }
+
     public function testEmptyShuffle()
     {
         $this->assertEquals([], Arr::shuffle([]));
@@ -1030,6 +1182,61 @@ class SupportArrTest extends TestCase
         $this->assertEquals($expect, Arr::sortRecursive($array));
     }
 
+    public function testSortRecursiveDesc()
+    {
+        $array = [
+            'empty' => [],
+            'nested' => [
+                'level1' => [
+                    'level2' => [
+                        'level3' => [2, 3, 1],
+                    ],
+                    'values' => [4, 5, 6],
+                ],
+            ],
+            'mixed' => [
+                'a' => 1,
+                2 => 'b',
+                'c' => 3,
+                1 => 'd',
+            ],
+            'numbered_index' => [
+                1 => 'e',
+                3 => 'c',
+                4 => 'b',
+                5 => 'a',
+                2 => 'd',
+            ],
+        ];
+
+        $expect = [
+            'empty' => [],
+            'mixed' => [
+                'c' => 3,
+                'a' => 1,
+                2 => 'b',
+                1 => 'd',
+            ],
+            'nested' => [
+                'level1' => [
+                    'values' => [6, 5, 4],
+                    'level2' => [
+                        'level3' => [3, 2, 1],
+                    ],
+                ],
+            ],
+            'numbered_index' => [
+                5 => 'a',
+                4 => 'b',
+                3 => 'c',
+                2 => 'd',
+                1 => 'e',
+            ],
+        ];
+
+        $this->assertEquals($expect, Arr::sortRecursiveDesc($array));
+    }
+
     public function testToCssClasses()
     {
         $classes = Arr::toCssClasses([
@@ -1261,17 +1468,24 @@ class SupportArrTest extends TestCase
         ], Arr::prependKeysWith($array, 'test.'));
     }
 
-    public function testTake()
+    public function testTake(): void
     {
         $array = [1, 2, 3, 4, 5, 6];
 
-        $this->assertEquals([
-            1, 2, 3,
-        ], Arr::take($array, 3));
+        // Test with a positive limit, should return the first 'limit' elements.
+        $this->assertEquals([1, 2, 3], Arr::take($array, 3));
 
-        $this->assertEquals([
-            4, 5, 6,
-        ], Arr::take($array, -3));
+        // Test with a negative limit, should return the last 'abs(limit)' elements.
+        $this->assertEquals([4, 5, 6], Arr::take($array, -3));
+
+        // Test with zero limit, should return an empty array.
+        $this->assertEquals([], Arr::take($array, 0));
+
+        // Test with a limit greater than the array size, should return the entire array.
+        $this->assertEquals([1, 2, 3, 4, 5, 6], Arr::take($array, 10));
+
+        // Test with a negative limit greater than the array size, should return the entire array.
+        $this->assertEquals([1, 2, 3, 4, 5, 6], Arr::take($array, -10));
     }
 
     public function testSelect()
@@ -1308,5 +1522,15 @@ class SupportArrTest extends TestCase
                 'name' => 'Abigail',
             ],
         ], Arr::select($array, 'name'));
+
+        $this->assertEquals([
+            [],
+            [],
+        ], Arr::select($array, 'nonExistingKey'));
+
+        $this->assertEquals([
+            [],
+            [],
+        ], Arr::select($array, null));
     }
 }
