@@ -5,7 +5,6 @@ namespace Illuminate\Tests\Support;
 use BadMethodCallException;
 use Carbon\Carbon as BaseCarbon;
 use Carbon\CarbonImmutable as BaseCarbonImmutable;
-use DateTime;
 use DateTimeInterface;
 use Illuminate\Support\Carbon;
 use PHPUnit\Framework\TestCase;
@@ -26,7 +25,7 @@ class SupportCarbonTest extends TestCase
 
     protected function tearDown(): void
     {
-        Carbon::setTestNow();
+        Carbon::setTestNow(null);
         Carbon::serializeUsing(null);
 
         parent::tearDown();
@@ -34,7 +33,7 @@ class SupportCarbonTest extends TestCase
 
     public function testInstance()
     {
-        $this->assertInstanceOf(DateTime::class, $this->now);
+        $this->assertInstanceOf(Carbon::class, $this->now);
         $this->assertInstanceOf(DateTimeInterface::class, $this->now);
         $this->assertInstanceOf(BaseCarbon::class, $this->now);
         $this->assertInstanceOf(Carbon::class, $this->now);
@@ -42,7 +41,7 @@ class SupportCarbonTest extends TestCase
 
     public function testCarbonIsMacroableWhenNotCalledStatically()
     {
-        Carbon::macro('diffInDecades', function (Carbon $dt = null, $abs = true) {
+        Carbon::macro('diffInDecades', function (?Carbon $dt = null, $abs = true) {
             return (int) ($this->diffInYears($dt, $abs) / 10);
         });
 
@@ -118,4 +117,28 @@ class SupportCarbonTest extends TestCase
         $this->assertSame('2017-06-27 13:14:15', BaseCarbon::now()->toDateTimeString());
         $this->assertSame('2017-06-27 13:14:15', BaseCarbonImmutable::now()->toDateTimeString());
     }
+
+    public function testCarbonIsConditionable()
+    {
+        $this->assertTrue(Carbon::now()->when(null, fn (Carbon $carbon) => $carbon->addDays(1))->isToday());
+        $this->assertTrue(Carbon::now()->when(true, fn (Carbon $carbon) => $carbon->addDays(1))->isTomorrow());
+    }
+
+    public function testCreateFromUid()
+    {
+        $ulid = Carbon::createFromId('01DXH9C4P0ED4AGJJP9CRKQ55C');
+        $this->assertEquals('2020-01-01 19:30:00.000000', $ulid->toDateTimeString('microsecond'));
+
+        $uuidv1 = Carbon::createFromId('71513cb4-f071-11ed-a0cf-325096b39f47');
+        $this->assertEquals('2023-05-12 03:02:34.147346', $uuidv1->toDateTimeString('microsecond'));
+
+        $uuidv2 = Carbon::createFromId('000003e8-f072-21ed-9200-325096b39f47');
+        $this->assertEquals('2023-05-12 03:06:33.529139', $uuidv2->toDateTimeString('microsecond'));
+
+        $uuidv6 = Carbon::createFromId('1edf0746-5d1c-6ce8-88ad-e0cb4effa035');
+        $this->assertEquals('2023-05-12 03:23:43.347428', $uuidv6->toDateTimeString('microsecond'));
+
+        $uuidv7 = Carbon::createFromId('01880dfa-2825-72e4-acbb-b1e4981cf8af');
+        $this->assertEquals('2023-05-12 03:21:18.117000', $uuidv7->toDateTimeString('microsecond'));
+    }
 }
