@@ -17,6 +17,7 @@ class ValidatedInputTest extends TestCase
 
         $this->assertSame('Taylor', $input->name);
         $this->assertSame('Taylor', $input['name']);
+        $this->assertEquals(['name' => 'Taylor'], $input->all(['name']));
         $this->assertEquals(['name' => 'Taylor'], $input->only(['name']));
         $this->assertEquals(['name' => 'Taylor'], $input->except(['votes']));
         $this->assertEquals(['name' => 'Taylor', 'votes' => 100], $input->all());
@@ -39,14 +40,14 @@ class ValidatedInputTest extends TestCase
     {
         $inputA = new ValidatedInput(['name' => 'Taylor']);
 
-        $this->assertEquals(true, $inputA->has('name'));
-        $this->assertEquals(true, $inputA->missing('votes'));
-        $this->assertEquals(true, $inputA->missing(['votes']));
-        $this->assertEquals(false, $inputA->missing('name'));
+        $this->assertTrue($inputA->has('name'));
+        $this->assertTrue($inputA->missing('votes'));
+        $this->assertTrue($inputA->missing(['votes']));
+        $this->assertFalse($inputA->missing('name'));
 
         $inputB = new ValidatedInput(['name' => 'Taylor', 'votes' => 100]);
 
-        $this->assertEquals(true, $inputB->has(['name', 'votes']));
+        $this->assertTrue($inputB->has(['name', 'votes']));
     }
 
     public function test_exists_method()
@@ -478,6 +479,20 @@ class ValidatedInputTest extends TestCase
         $this->assertNull($input->enum('invalid_enum_value', StringBackedEnum::class));
     }
 
+    public function test_enums_method()
+    {
+        $input = new ValidatedInput([
+            'valid_enum_value' => 'Hello world',
+            'invalid_enum_value' => 'invalid',
+        ]);
+
+        $this->assertEmpty($input->enums('doesnt_exists', StringBackedEnum::class));
+
+        $this->assertEquals([StringBackedEnum::HELLO_WORLD], $input->enums('valid_enum_value', StringBackedEnum::class));
+
+        $this->assertEmpty($input->enums('invalid_enum_value', StringBackedEnum::class));
+    }
+
     public function test_collect_method()
     {
         $input = new ValidatedInput(['users' => [1, 2, 3]]);
