@@ -3,13 +3,12 @@
 namespace Illuminate\Tests\Translation;
 
 use Illuminate\Translation\MessageSelector;
+use PHPUnit\Framework\Attributes\DataProvider;
 use PHPUnit\Framework\TestCase;
 
 class TranslationMessageSelectorTest extends TestCase
 {
-    /**
-     * @dataProvider chooseTestData
-     */
+    #[DataProvider('chooseTestData')]
     public function testChoose($expected, $id, $number)
     {
         $selector = new MessageSelector;
