@@ -2,8 +2,10 @@
 
 namespace Illuminate\Tests\View\Blade;
 
+use Illuminate\Container\Container;
 use Illuminate\Filesystem\Filesystem;
 use Illuminate\View\Compilers\BladeCompiler;
+use Illuminate\View\Component;
 use Mockery as m;
 use PHPUnit\Framework\TestCase;
 
@@ -16,12 +18,18 @@ abstract class AbstractBladeTestCase extends TestCase
 
     protected function setUp(): void
     {
-        $this->compiler = new BladeCompiler($this->getFiles(), __DIR__);
         parent::setUp();
+
+        $this->compiler = new BladeCompiler($this->getFiles(), __DIR__);
     }
 
     protected function tearDown(): void
     {
+        Container::setInstance(null);
+        Component::flushCache();
+        Component::forgetComponentsResolver();
+        Component::forgetFactory();
+
         m::close();
 
         parent::tearDown();
