@@ -9,5 +9,9 @@ class BladeUnsetStatementsTest extends AbstractBladeTestCase
         $string = '@unset ($unset)';
         $expected = '<?php unset($unset); ?>';
         $this->assertEquals($expected, $this->compiler->compileString($string));
+
+        $string = '@unset ($unset)))';
+        $expected = '<?php unset($unset); ?>))';
+        $this->assertEquals($expected, $this->compiler->compileString($string));
     }
 }
