@@ -14,16 +14,16 @@ assertType('mixed', $cache->get('cache', function (): int {
 }));
 
 assertType('mixed', $cache->pull('key'));
-assertType('int', $cache->pull('cache', 28));
-assertType('int', $cache->pull('cache', function (): int {
+assertType('28', $cache->pull('cache', 28));
+assertType('30', $cache->pull('cache', function (): int {
     return 30;
 }));
-assertType('int', $cache->sear('cache', function (): int {
+assertType('33', $cache->sear('cache', function (): int {
     return 33;
 }));
-assertType('int', $cache->remember('cache', now(), function (): int {
+assertType('36', $cache->remember('cache', now(), function (): int {
     return 36;
 }));
-assertType('int', $cache->rememberForever('cache', function (): int {
+assertType('36', $cache->rememberForever('cache', function (): int {
     return 36;
 }));
